using System;
using System.Collections.Generic;
using System.Text;

namespace SmartMathLibrary
{
    /// <summary>
    /// This class represents a mathematical arithmetical progression.
    /// </summary>
    [Serializable]
    public class ArithmeticalSequence
    {
        /// <summary>
        /// Represents the biggest index of the arithmetical progression.
        /// </summary>
        private int lastIndex;

        /// <summary>
        /// Represents the start value of the arithmetical progression.
        /// </summary>
        private double startValue;

        /// <summary>
        /// Represents the constant difference between two elements.
        /// </summary>
        private double difference;

        /// <summary>
        /// Includes the numeric data for the arithmetical progression.
        /// </summary>
        private double[] progressionData;

        /// <summary>
        /// Initializes a new instance of the <see cref="ArithmeticalSequence"/> class.
        /// </summary>
        /// <param name="startValue">The start value (a0) of the arithmetical progression.</param>
        /// <param name="lastIndex">The biggest index of the arithmetical progression.</param>
        /// <param name="difference">The constant difference between two elements.</param>
        public ArithmeticalSequence(double startValue, int lastIndex, double difference)
        {
            if (lastIndex < 0)
            {
                throw new ArgumentException("lastIndex < 0", "lastIndex");
            }

            this.lastIndex = lastIndex;
            this.startValue = startValue;
            this.difference = difference;
            this.progressionData = new double[0];
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ArithmeticalSequence"/> class.
        /// </summary>
        /// <param name="difference">The constant difference between two elements.</param>
        /// <param name="progressionData">The progression data in a double array.</param>
        public ArithmeticalSequence(double difference, double[] progressionData)
        {
            if (progressionData == (double[]) ((object) null))
            {
                throw new ArgumentNullException("progressionData");
            }
            if (progressionData.Length == 0)
            {
                throw new ArgumentException("progressionData.Length == 0", "progressionData");
            }

            this.lastIndex = progressionData.Length - 1;
            this.startValue = progressionData[0];
            this.difference = difference;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ArithmeticalSequence"/> class.
        /// </summary>
        /// <param name="progressionData">The progression data in a double array. The difference value 
        /// will be automatically calculated out of the first two elements.</param>
        public ArithmeticalSequence(double[] progressionData)
        {
            if (!ArithmeticalSequence.IsArithmeticalSequence(progressionData))
            {
                throw new SequenceException("The numerical data does not represents an arithmetical progression");
            }

            this.lastIndex = progressionData.Length;
            this.startValue = progressionData[0];
            this.difference = progressionData[1] - progressionData[0];
        }

        /// <summary>
        /// Gets or sets the biggest index of the arithmetical progression.
        /// </summary>
        /// <value>The last index.</value>
        public int LastIndex
        {
            get { return lastIndex; }
            set { lastIndex = value; }
        }

        /// <summary>
        /// Gets or sets the start value of the arithmetical progression.
        /// </summary>
        /// <value>The start value.</value>
        public double StartValue
        {
            get { return startValue; }
            set { startValue = value; }
        }

        /// <summary>
        /// Gets or sets the difference of the elements.
        /// </summary>
        /// <value>The difference.</value>
        public double Difference
        {
            get { return difference; }
            set { difference = value; }
        }

        /// <summary>
        /// Gets the numeric data of the progression arithmetical sequence.
        /// </summary>
        /// <value>The numeric data of the progression arithmetical sequence.</value>
        public double[] ProgressionData
        {
            get { return progressionData; }
        }

        /// <summary>
        /// Computes the arithmetical progression out of the given values. This method has to be called
        /// separate, because sometimes the calculation may take a long time.
        /// </summary>
        public void ComputeProgression()
        {
            double tempuri = startValue;
            this.progressionData = new double[this.lastIndex];

            for (int i = 0; i < this.lastIndex; i++)
            {
                this.progressionData[i] = tempuri;
                tempuri += this.difference;
            }
        }

        /// <summary>
        /// Calculates the sum of all elements in the arithmetical progression.
        /// </summary>
        /// <returns>The sum of the whole progression (s).</returns>
        public double CalculateSum()
        {
            return (this.lastIndex * (2 * this.startValue + ((this.lastIndex - 1) * this.difference))) / 2;
        }

        /// <summary>
        /// Calculates the arithmetic average of the sequence.
        /// </summary>
        /// <returns>The arithmetic average of the sequence.</returns>
        public double CalculateArithmeticAverage()
        {
            return this.CalculateSum() / (double) this.lastIndex;
        }

        /// <summary>
        /// Determines whether [is arithmetical progression] [the specified progression data].
        /// </summary>
        /// <param name="progressionData">The progression data to check.</param>
        /// <returns>
        /// 	<c>true</c> if [is an arithmetical progression] [the specified progression data]; otherwise, <c>false</c>.
        /// </returns>
        public static bool IsArithmeticalSequence(double[] progressionData)
        {
            if (progressionData == (double[]) ((object) null))
            {
                throw new ArgumentNullException("progressionData");
            }

            if (progressionData.Length <= 2)
            {
                return false;
            }

            double check = progressionData[1] - progressionData[0];

            for (int i = 1; i < progressionData.Length; i++)
            {
                if (progressionData[i - 1] + check != progressionData[i])
                {
                    return false;
                }
            }

            return true;
        }

        /// <summary>
        /// Determines whether the specified <see cref="T:System.Object"/> is equal to the current <see cref="T:System.Object"/>.
        /// </summary>
        /// <param name="obj">The <see cref="T:System.Object"/> to compare with the current <see cref="T:System.Object"/>.</param>
        /// <returns>
        /// true if the specified <see cref="T:System.Object"/> is equal to the current <see cref="T:System.Object"/>; otherwise, false.
        /// </returns>
        /// <exception cref="T:System.NullReferenceException">The <paramref name="obj"/> parameter is null.</exception>
        public override bool Equals(object obj)
        {
            return base.Equals(obj);
        }

        /// <summary>
        /// Serves as a hash function for a particular type.
        /// </summary>
        /// <returns>
        /// A hash code for the current <see cref="T:System.Object"/>.
        /// </returns>
        public override int GetHashCode()
        {
            return base.GetHashCode();
        }

        /// <summary>
        /// Returns a <see cref="T:System.String"/> that represents the current <see cref="T:System.Object"/>.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.String"/> that represents the current <see cref="T:System.Object"/>.
        /// </returns>
        public override string ToString()
        {
            String result = "";

            for (int i = 0; i < this.progressionData.Length; i++)
            {
                result += this.progressionData[i] + "; ";
            }

            return result;
        }
    }
}