﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SmartMathLibrary
{
    /// <summary>
    /// This class provides the approximation of the derivative of a specified function by using 
    /// a small limiting value.
    /// </summary>
    [Serializable]
    public class ComplexDerivativeApproximator
    {
        /// <summary>
        /// The source function for the to approximate derivative.
        /// </summary>
        private IComplexFunction sourceFunction;

        /// <summary>
        /// Initializes a new instance of the <see cref="ComplexDerivativeApproximator"/> class.
        /// </summary>
        /// <param name="sourceFunction">The source function for the to approximate derivative.</param>
        public ComplexDerivativeApproximator(IComplexFunction sourceFunction)
        {
            this.sourceFunction = sourceFunction;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ComplexDerivativeApproximator"/> class.
        /// </summary>
        /// <param name="sourceFunction">The source function for the to approximate derivative.</param>
        public ComplexDerivativeApproximator(ComplexPolynomial sourceFunction)
        {
            this.sourceFunction = sourceFunction;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ComplexDerivativeApproximator"/> class.
        /// </summary>
        /// <param name="sourceFunction">The source function for the to approximate derivative.</param>
        public ComplexDerivativeApproximator(ComplexRationalPolynomial sourceFunction)
        {
            if (sourceFunction == (ComplexRationalPolynomial) null)
            {
                throw new ArgumentNullException("sourceFunction");
            }

            this.sourceFunction = sourceFunction;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ComplexDerivativeApproximator"/> class.
        /// </summary>
        /// <param name="sourceFunction">The source function for the to approximate derivative.</param>
        public ComplexDerivativeApproximator(HardComplexFunction sourceFunction)
        {
            this.sourceFunction = sourceFunction;
        }

        /// <summary>
        /// Gets or sets the source function for the to approximate derivative.
        /// </summary>
        /// <value>The source function for the to approximate derivative.</value>
        public IComplexFunction SourceFunction
        {
            get { return sourceFunction; }
            set { sourceFunction = value; }
        }

        /// <summary>
        /// Approximates the specified complex function derivative by using a small limiting value.
        /// </summary>
        /// <param name="x">The x at which the function derivative should be calculate.</param>
        /// <returns>The specified slope at the position x.</returns>
        public ComplexNumber LimitingValueAnalyse(ComplexNumber x)
        {
            return this.LimitingValueAnalyse(x, 1e-5);
        }

        /// <summary>
        /// Approximates the specified complex function derivative by using a small limiting value.
        /// </summary>
        /// <param name="x">The x at which the function derivative should be calculate.</param>
        /// <param name="epsilon">The epsilon of the small limiting value.</param>
        /// <returns>The specified slope at the position x.</returns>
        public ComplexNumber LimitingValueAnalyse(ComplexNumber x, double epsilon)
        {
            if (x == (ComplexNumber) null)
            {
                throw new ArgumentException("x");
            }

            return (this.sourceFunction.SolveAt(x + epsilon) - this.sourceFunction.SolveAt(x))
                   / epsilon;
        }
    }
}