﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SmartMathLibrary
{
    /// <summary>
    /// This class represents a complex linear factor in the form of (x - (x0-i)).
    /// </summary>
    [Serializable]
    public class ComplexLinearFactor : IComplexLinearFactor
    {
        /// <summary>
        /// The summate of the linear factor.
        /// </summary>
        private ComplexNumber summat;

        /// <summary>
        /// The variable name of the linear factor.
        /// </summary>
        private string variableName;

        /// <summary>
        /// Initializes a new instance of the <see cref="ComplexLinearFactor"/> class.
        /// </summary>
        /// <param name="summat">The summate of the linear factor.</param>
        public ComplexLinearFactor(ComplexNumber summat)
        {
            this.summat = summat.Copy();
            this.variableName = "x";
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ComplexLinearFactor"/> class.
        /// </summary>
        /// <param name="factor">The linear factor to clone.</param>
        public ComplexLinearFactor(ComplexLinearFactor factor)
        {
            if (factor == (ComplexLinearFactor) null)
            {
                throw new ArgumentNullException("factor");
            }

            this.summat = factor.Summat.Copy();
            this.variableName = factor.VariableName;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ComplexLinearFactor"/> class.
        /// </summary>
        /// <param name="summat">The summate of the linear factor.</param>
        /// <param name="variableName">The variable name of the linear factor.</param>
        public ComplexLinearFactor(ComplexNumber summat, string variableName)
        {
            this.summat = summat.Copy();
            this.variableName = variableName;
        }

        /// <summary>
        /// Gets or sets the summate of the linear factor.
        /// </summary>
        /// <value>The summate of the linear factor.</value>
        public ComplexNumber Summat
        {
            get { return summat; }
            set { summat = value; }
        }

        /// <summary>
        /// Gets or sets the name of the variable.
        /// </summary>
        /// <value>The name of the variable.</value>
        public string VariableName
        {
            get { return variableName; }
            set { variableName = value; }
        }

        /// <summary>
        /// Converts the linear factor to a polynomial.
        /// </summary>
        /// <returns>The converted linear factor.</returns>
        public ComplexPolynomial ToPolynomial()
        {
            ComplexNumber tempuri = this.summat.Copy();
            ComplexPolynomial result = new ComplexPolynomial(2);

            tempuri.Invert();

            result.SetComponentAt(0, new ComplexPolynomialComponent(tempuri, 0));
            result.SetComponentAt(0, new ComplexPolynomialComponent(new ComplexNumber(1), 1));

            return result;
        }

        /// <summary>
        /// Implements the operator ==.
        /// </summary>
        /// <param name="a">The first linear factor.</param>
        /// <param name="b">The second linear factor.</param>
        /// <returns>The result of the operator.</returns>
        public static bool operator ==(ComplexLinearFactor a, ComplexLinearFactor b)
        {
            if (System.Object.ReferenceEquals(a, b))
            {
                return true;
            }

            if (((object) a == null) || ((object) b == null))
            {
                return false;
            }

            return ((a.Summat == b.Summat) && (a.VariableName.Equals(b.VariableName)));
        }

        /// <summary>
        /// Implements the operator !=.
        /// </summary>
        /// <param name="a">The first linear factor.</param>
        /// <param name="b">The second linear factor.</param>
        /// <returns>The result of the operator.</returns>
        public static bool operator !=(ComplexLinearFactor a, ComplexLinearFactor b)
        {
            return !(a == b);
        }

        /// <summary>
        /// Compares the current instance of a linear factor to another.
        /// </summary>
        /// <param name="factor">The factor.</param>
        /// <returns>True if the two factors are even otherwise, false.</returns>
        public bool CompareTo(ComplexLinearFactor factor)
        {
            if (factor == (ComplexLinearFactor) null)
            {
                throw new ArgumentNullException("factor");
            }

            return this == factor;
        }

        /// <summary>
        /// Determines whether the specified <see cref="T:System.Object"/> is equal to the current <see cref="T:System.Object"/>.
        /// </summary>
        /// <param name="obj">The <see cref="T:System.Object"/> to compare with the current <see cref="T:System.Object"/>.</param>
        /// <returns>
        /// true if the specified <see cref="T:System.Object"/> is equal to the current <see cref="T:System.Object"/>; otherwise, false.
        /// </returns>
        /// <exception cref="T:System.NullReferenceException">The <paramref name="obj"/> parameter is null.</exception>
        public override bool Equals(object obj)
        {
            return base.Equals(obj);
        }

        /// <summary>
        /// Serves as a hash function for a particular type.
        /// </summary>
        /// <returns>
        /// A hash code for the current <see cref="T:System.Object"/>.
        /// </returns>
        public override int GetHashCode()
        {
            return base.GetHashCode();
        }

        /// <summary>
        /// Returns a <see cref="T:System.String"/> that represents the current <see cref="T:System.Object"/>.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.String"/> that represents the current <see cref="T:System.Object"/>.
        /// </returns>
        public override string ToString()
        {
            String result = "(" + variableName;

            if (this.summat.RealNumberPart < 0)
            {
                result += " - ";
            }
            else
            {
                result += " + ";
            }

            result += ExtendedMath.Fabs(this.summat) + ")";

            return result;
        }
    }
}