﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SmartMathLibrary
{
    /// <summary>
    /// This class represents a cubic spline with n splineparts.
    /// </summary>
    [Serializable]
    public class CubicSpline : AbstractSpline, ICubicSpline
    {
        /// <summary>
        /// Ths list of splineparts.
        /// </summary>
        private readonly List<CubicSplineComponent> components;

        /// <summary>
        /// Initializes a new instance of the <see cref="CubicSpline"/> class.
        /// </summary>
        public CubicSpline()
        {
            this.components = new List<CubicSplineComponent>();
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="CubicSpline"/> class.
        /// </summary>
        /// <param name="components">The components or splineparts of the spline.</param>
        public CubicSpline(CubicSplineComponent[] components)
        {
            this.components = new List<CubicSplineComponent>();

            foreach (CubicSplineComponent c in components)
            {
                this.components.Add(c.Copy());
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="CubicSpline"/> class.
        /// </summary>
        /// <param name="spline">The cubic spline to clone.</param>
        public CubicSpline(CubicSpline spline)
        {
            if (spline == (CubicSpline) null)
            {
                throw new ArgumentNullException("spline");
            }

            this.components = new List<CubicSplineComponent>();

            foreach (CubicSplineComponent c in spline.Components)
            {
                this.components.Add(c.Copy());
            }
        }

        /// <summary>
        /// Gets or sets the components or parts of the spline.
        /// </summary>
        /// <value>The components or parts of the spline.</value>
        public List<CubicSplineComponent> Components
        {
            get { return components; }
        }

        /// <summary>
        /// Gets the number of splineparts.
        /// </summary>
        /// <value>The number of splineparts.</value>
        public int Count
        {
            get { return components.Count; }
        }

        /// <summary>
        /// Adds a specified splinepart to the current spline.
        /// </summary>
        /// <param name="component">The splinepart to add.</param>
        public void Add(CubicSplineComponent component)
        {
            if (component == (CubicSplineComponent) null)
            {
                throw new ArgumentNullException("component");
            }

            this.components.Add(component.Copy());
        }

        /// <summary>
        /// Removes the splinepart at the specified index.
        /// </summary>
        /// <param name="index">The specified index.</param>
        public void RemoveAt(int index)
        {
            this.components.RemoveAt(index);
        }

        /// <summary>
        /// Removes all splineparts.
        /// </summary>
        public void Clear()
        {
            this.components.Clear();
        }

        /// <summary>
        /// Calculates the value of the spline at the specified position x.
        /// </summary>
        /// <param name="x">The specified position x.</param>
        /// <returns>The s(x) of the spline.</returns>
        public override double SolveAt(double x)
        {
            foreach (CubicSplineComponent component in this.components)
            {
                if (component.IsDefinedFor(x))
                {
                    return component.Polynomial.SolveAt(x);
                }
            }

            throw new DomainNotDefinedException("The current spline is not defined for this x value");
        }

        /// <summary>
        /// Determines whether [the current spline is defined for] [the specified x value].
        /// </summary>
        /// <param name="x">The specified x value, which should be checked.</param>
        /// <returns>
        /// 	<c>true</c> if [the current spline is defined for] [the specified x value];
        /// 	otherwise, <c>false</c>.
        /// </returns>
        public bool IsDefinedFor(double x)
        {
            foreach (CubicSplineComponent component in this.components)
            {
                if (component.IsDefinedFor(x))
                {
                    return true;
                }
            }

            return false;
        }

        /// <summary>
        /// Creates a copy of the current instance.
        /// </summary>
        /// <returns>A copy of the current instance.</returns>
        public CubicSpline Copy()
        {
            return new CubicSpline((CubicSplineComponent[]) this.components.ToArray().Clone());
        }

        /// <summary>
        /// Gets the enumerator.
        /// </summary>
        /// <returns>The enumerator of the current iteration.</returns>
        public IEnumerator<CubicSplineComponent> GetEnumerator()
        {
            for (int i = 0; i < this.components.Count; i++)
            {
                yield return this.components[i];
            }
        }

        /// <summary>
        /// Compares the current instance of a cubic spline to another one. 
        /// The order of the splineparts, which the spline includes doesn´t matter.
        /// </summary>
        /// <param name="compare">The cubicspline to compare.</param>
        /// <returns>True if the splines are even otherwise, false.</returns>
        public bool CompareTo(CubicSpline compare)
        {
            if (compare == (CubicSpline) null)
            {
                throw new ArgumentNullException("compare");
            }

            foreach (CubicSplineComponent c in compare)
            {
                if (!this.ComparePartTo(c))
                {
                    return false;
                }
            }

            return true;
        }

        /// <summary>
        /// Checks if the current instance contains a specified CubicSplineComponent.
        /// </summary>
        /// <param name="compare">The specified CubicSplineComponent to check.</param>
        /// <returns>True if the current instance contains the specified CubicSplineComponent 
        /// otherwise, false.</returns>
        private bool ComparePartTo(CubicSplineComponent compare)
        {
            foreach (CubicSplineComponent c in this.components)
            {
                if (c.CompareTo(compare))
                {
                    return true;
                }
            }

            return false;
        }
    }
}