﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using SmartMathLibrary;

namespace SmartMathLibrary
{
    /// <summary>
    /// This class represents a vector of quaternions.
    /// </summary>
    [Serializable]
    public class GeneralQuaternionVector : AbstractGeneralVector, IGeneralQuaternionVector
    {
        /// <summary>
        /// Saves the components of the quaternion vector.
        /// </summary>
        private Quaternion[] vectorData;

        /// <summary>
        /// Initializes a new instance of the <see cref="GeneralQuaternionVector"/> class.
        /// </summary>
        /// <param name="vectorData">The vector to clone.</param>
        public GeneralQuaternionVector(GeneralQuaternionVector vectorData)
        {
            if (vectorData == (GeneralQuaternionVector) null)
            {
                throw new ArgumentNullException("vectorData");
            }

            this.VectorData = (Quaternion[]) (vectorData.VectorData.Clone());
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="GeneralQuaternionVector"/> class.
        /// </summary>
        /// <param name="data">The components of the vector.</param>
        public GeneralQuaternionVector(Quaternion[] data)
        {
            this.VectorData = data;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="GeneralQuaternionVector"/> class.
        /// </summary>
        /// <param name="dimension">The dimension of the vector.</param>
        public GeneralQuaternionVector(int dimension)
        {
            if (dimension < 0)
            {
                throw new ArgumentException("dimension < 0", "dimension");
            }

            this.vectorData = new Quaternion[dimension];

            for (int i = 0; i < dimension; i++)
            {
                this.vectorData[i] = new Quaternion();
            }
        }

        /// <summary>
        /// Gets or sets the components of the quaternion vector.
        /// </summary>
        /// <value>The components of the quaternion vector.</value>
        public Quaternion[] VectorData
        {
            get { return vectorData; }
            set { vectorData = value; }
        }

        /// <summary>
        /// Counts the components of the quaternion vector.
        /// </summary>
        /// <value>The components of the quaternion vector.</value>
        public int Count
        {
            get { return vectorData.Length; }
        }

        /// <summary>
        /// Gets the length of the vector.
        /// </summary>
        /// <value>The length of the vector.</value>
        public double Length
        {
            get
            {
                double result = 0;

                for (int i = 0; i < this.Count; i++)
                {
                    result += Math.Pow(this[i].Length, 2);
                }

                return Math.Sqrt(result);
            }
        }

        /// <summary>
        /// Gets or sets the <see cref="SmartMathLibrary.Quaternion"/> at the specified index.
        /// </summary>
        /// <value>The quaternion component of the vector at the specified position.</value>
        public Quaternion this[int index]
        {
            get { return this.GetComponent(index); }
            set { this.SetComponent(index, value); }
        }

        /// <summary>
        /// Extracts the real number part of the quaternion values into a new vector.
        /// </summary>
        /// <returns>The real number part of the quaternion values.</returns>
        public GeneralVector ExtractRealNumberPartVector()
        {
            GeneralVector result = new GeneralVector(this.Count);

            for (int i = 0; i < this.Count; i++)
            {
                result[i] = this[i].X0;
            }

            return result;
        }

        /// <summary>
        /// Extracts the i number part of the quaternion values into a new vector.
        /// </summary>
        /// <returns>The i number part of the quaternion values.</returns>
        public GeneralVector ExtractINumberPartVector()
        {
            GeneralVector result = new GeneralVector(this.Count);

            for (int i = 0; i < this.Count; i++)
            {
                result[i] = this[i].I;
            }

            return result;
        }

        /// <summary>
        /// Extracts the j number part of the quaternion values into a new vector.
        /// </summary>
        /// <returns>The j number part of the quaternion values.</returns>
        public GeneralVector ExtractJNumberPartVector()
        {
            GeneralVector result = new GeneralVector(this.Count);

            for (int i = 0; i < this.Count; i++)
            {
                result[i] = this[i].J;
            }

            return result;
        }

        /// <summary>
        /// Extracts the k number part of the quaternion values into a new vector.
        /// </summary>
        /// <returns>The k number part of the quaternion values.</returns>
        public GeneralVector ExtractKNumberPartVector()
        {
            GeneralVector result = new GeneralVector(this.Count);

            for (int i = 0; i < this.Count; i++)
            {
                result[i] = this[i].K;
            }

            return result;
        }

        /// <summary>
        /// Gets the component value.
        /// </summary>
        /// <param name="index">The index of the component.</param>
        /// <returns>The vector component value at the over given index</returns>
        public Quaternion GetComponent(int index)
        {
            return this.vectorData[index];
        }

        /// <summary>
        /// Sets the component value at a specified position.
        /// </summary>
        /// <param name="index">The index of the component.</param>
        /// <param name="value">The new value of the component.</param>
        public void SetComponent(int index, Quaternion value)
        {
            this.vectorData[index] = value;
        }

        /// <summary>
        /// Counts the components of the vector.
        /// </summary>
        /// <value>The number of components of the vector.</value>
        public GeneralQuaternionVector Copy()
        {
            GeneralQuaternionVector result = new GeneralQuaternionVector(this.Count);

            for (int i = 0; i < this.vectorData.Length; i++)
            {
                result[i] = this[i].Copy();
            }

            return result;
        }

        ///// <summary>
        ///// Converts the current instance of the vector to a row matrix.
        ///// </summary>
        ///// <returns>The converted matrix.</returns>
        //public ComplexMatrix ToMatrix()
        //{
        //    ComplexMatrix matrix = new ComplexMatrix(this.vectorData.Length, 1);

        //    for (int i = 0; i < this.vectorData.Length; i++)
        //    {
        //        matrix.SetValueAtPosition(new MatrixPosition(i, 0), this.vectorData[i]);
        //    }

        //    return matrix;
        //}

        /// <summary>
        /// Converts the data of the vector to an array of quaternions.
        /// </summary>
        /// <returns>An array of quaternions.</returns>
        public Quaternion[] ToArray()
        {
            return (Quaternion[]) this.vectorData.Clone();
        }

        /// <summary>
        /// Implements the operator ==.
        /// </summary>
        /// <param name="a">The first quaternion vector.</param>
        /// <param name="b">The second quaternion vector.</param>
        /// <returns>The result of the operator.</returns>
        public static bool operator ==(GeneralQuaternionVector a, GeneralQuaternionVector b)
        {
            if (System.Object.ReferenceEquals(a, b))
            {
                return true;
            }

            if (((object) a == null) || ((object) b == null))
            {
                return false;
            }

            if (a.Count != b.Count)
            {
                return false;
            }

            for (int i = 0; i < a.Count; i++)
            {
                if (a.GetComponent(i) != b.GetComponent(i))
                {
                    return false;
                }
            }

            return true;
        }

        /// <summary>
        /// Implements the operator !=.
        /// </summary>
        /// <param name="a">The first quaternion vector.</param>
        /// <param name="b">The second quaternion vector.</param>
        /// <returns>The result of the operator.</returns>
        public static bool operator !=(GeneralQuaternionVector a, GeneralQuaternionVector b)
        {
            return !(a == b);
        }

        /// <summary>
        /// Unions the specified two quaternion vectors.
        /// </summary>
        /// <param name="a">The first quaternion vector to union.</param>
        /// <param name="b">The second quaternion vector to union.</param>
        /// <returns>The two source vector values in one vector.</returns>
        public static GeneralQuaternionVector UnionVectors(GeneralQuaternionVector a, GeneralQuaternionVector b)
        {
            if (a == (GeneralQuaternionVector) null)
            {
                throw new ArgumentNullException("a");
            }

            if (b == (GeneralQuaternionVector) null)
            {
                throw new ArgumentNullException("b");
            }

            GeneralQuaternionVector result = new GeneralQuaternionVector(a.Count + b.Count);

            for (int i = 0; i < a.Count; i++)
            {
                result[i] = a[i];
            }

            int add = a.Count;

            for (int i = 0; i < b.Count; i++)
            {
                result[i + add] = b[i];
            }

            return result;
        }

        /// <summary>
        /// Computes the dot product of two vectors.
        /// </summary>
        /// <param name="a">The first vector.</param>
        /// <param name="b">The second vector.</param>
        /// <returns>The dot product of both vectors.</returns>
        public static Quaternion ComputeDotProduct(GeneralQuaternionVector a, GeneralQuaternionVector b)
        {
            if (a == (GeneralQuaternionVector) null)
            {
                throw new ArgumentNullException("a");
            }

            if (b == (GeneralQuaternionVector) null)
            {
                throw new ArgumentNullException("b");
            }

            if (a.Count != b.Count)
            {
                throw new ArithmeticException("The number of components has to be even to compute the dot product.");
            }

            Quaternion tempuri = new Quaternion();

            for (int i = 0; i < a.Count; i++)
            {
                tempuri += a[i] * b[i];
            }

            return tempuri;
        }

        /// <summary>
        /// Implements the operator +.
        /// </summary>
        /// <param name="a">The first vector.</param>
        /// <param name="b">The second vector.</param>
        /// <returns>The result of the operator.</returns>
        public static GeneralQuaternionVector operator +(GeneralQuaternionVector a, GeneralQuaternionVector b)
        {
            if (a == (GeneralQuaternionVector) null)
            {
                throw new ArgumentNullException("a");
            }

            if (b == (GeneralQuaternionVector) null)
            {
                throw new ArgumentNullException("b");
            }

            if (a.Count != b.Count)
            {
                throw new ArithmeticException("Number of components in both vectors have to be even.");
            }

            Quaternion[] result = new Quaternion[a.Count];

            for (int i = 0; i < a.Count; i++)
            {
                result[i] = a.GetComponent(i) + b.GetComponent(i);
            }

            return new GeneralQuaternionVector(result);
        }

        /// <summary>
        /// Implements the operator -.
        /// </summary>
        /// <param name="a">The first vector.</param>
        /// <param name="b">The second vector.</param>
        /// <returns>The result of the operator.</returns>
        public static GeneralQuaternionVector operator -(GeneralQuaternionVector a, GeneralQuaternionVector b)
        {
            if (a == (GeneralQuaternionVector) null)
            {
                throw new ArgumentNullException("a");
            }

            if (b == (GeneralQuaternionVector) null)
            {
                throw new ArgumentNullException("b");
            }

            if (a.Count != b.Count)
            {
                throw new ArithmeticException("Number of components in both vectors have to be even.");
            }

            Quaternion[] result = new Quaternion[a.Count];

            for (int i = 0; i < a.Count; i++)
            {
                result[i] = a.GetComponent(i) - b.GetComponent(i);
            }

            return new GeneralQuaternionVector(result);
        }

        /// <summary>
        /// Implements the operator *.
        /// </summary>
        /// <param name="a">The vector for the multipliaction.</param>
        /// <param name="value">The scale value to multiply.</param>
        /// <returns>The result of the operator.</returns>
        public static GeneralQuaternionVector operator *(GeneralQuaternionVector a, Quaternion value)
        {
            if (a == (GeneralQuaternionVector) null)
            {
                throw new ArgumentNullException("a");
            }

            Quaternion[] result = new Quaternion[a.Count];

            for (int i = 0; i < a.Count; i++)
            {
                result[i] = a.GetComponent(i) * value;
            }

            return new GeneralQuaternionVector(result);
        }

        /// <summary>
        /// Determines whether the specified <see cref="T:System.Object"/> is equal to the current <see cref="T:System.Object"/>.
        /// </summary>
        /// <param name="obj">The <see cref="T:System.Object"/> to compare with the current <see cref="T:System.Object"/>.</param>
        /// <returns>
        /// true if the specified <see cref="T:System.Object"/> is equal to the current <see cref="T:System.Object"/>; otherwise, false.
        /// </returns>
        /// <exception cref="T:System.NullReferenceException">The <paramref name="obj"/> parameter is null.</exception>
        public override bool Equals(object obj)
        {
            return base.Equals(obj);
        }

        /// <summary>
        /// Serves as a hash function for a particular type.
        /// </summary>
        /// <returns>
        /// A hash code for the current <see cref="T:System.Object"/>.
        /// </returns>
        public override int GetHashCode()
        {
            return base.GetHashCode();
        }

        /// <summary>
        /// Returns a <see cref="T:System.String"/> that represents the current <see cref="T:System.Object"/>.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.String"/> that represents the current <see cref="T:System.Object"/>.
        /// </returns>
        public override string ToString()
        {
            String result = "";

            for (int i = 0; i < this.vectorData.Length; i++)
            {
                result += this.vectorData[i] + "; ";
            }

            if (result.Length > 2)
            {
                result = result.Substring(0, result.Length - 2);
            }

            return result;
        }
    }
}