﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SmartMathLibrary
{
    /// <summary>
    /// This class represents a mathematical vector with 2 components. It should be primary used
    /// for geometric problems in a plane space.
    /// </summary>
    [Serializable]
    public class GeneralVector2D : GeneralVector
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="GeneralVector2D"/> class.
        /// </summary>
        public GeneralVector2D()
            : base(2)
        {
            this.SetComponent(0, 0);
            this.SetComponent(1, 0);
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="GeneralVector2D"/> class.
        /// </summary>
        /// <param name="vector">The vector which should converted to a Vector2D. Only the first
        /// two components of the specified vector will be set. If the vector includes more
        /// elements they will be thrown away.</param>
        public GeneralVector2D(GeneralVector vector)
            : base(2)
        {
            if (vector == (GeneralVector) null)
            {
                throw new ArgumentNullException("vector");
            }

            this.SetComponent(0, vector.GetComponent(0));
            this.SetComponent(1, vector.GetComponent(1));
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="GeneralVector2D"/> class.
        /// </summary>
        /// <param name="vector">Another GeneralVector2D instance that should be cloned.</param>
        public GeneralVector2D(GeneralVector2D vector)
            : base(2)
        {
            if (vector == (GeneralVector2D) null)
            {
                throw new ArgumentNullException("vector");
            }

            this.SetComponent(0, vector.GetComponent(0));
            this.SetComponent(1, vector.GetComponent(1));
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="GeneralVector2D"/> class.
        /// </summary>
        /// <param name="x">The first component of the vector(x).</param>
        /// <param name="y">The second component of the vector(y).</param>
        public GeneralVector2D(double x, double y)
            : base(2)
        {
            this.SetComponent(0, x);
            this.SetComponent(1, y);
        }

        /// <summary>
        /// Gets or sets the x value of the GeneralVector2D.
        /// </summary>
        /// <value>The x value of the GeneralVector2D.</value>
        public double X
        {
            get { return this.GetComponent(0); }
            set { this.SetComponent(0, value); }
        }

        /// <summary>
        /// Gets or sets the y value of the GeneralVector2D.
        /// </summary>
        /// <value>The y value of the GeneralVector2D.</value>
        public double Y
        {
            get { return this.GetComponent(1); }
            set { this.SetComponent(1, value); }
        }

        /// <summary>
        /// Implements the operator +.
        /// </summary>
        /// <param name="a">The first GeneralVector2D.</param>
        /// <param name="b">The second GeneralVector2D.</param>
        /// <returns>The result of the operator.</returns>
        public static GeneralVector2D operator +(GeneralVector2D a, GeneralVector2D b)
        {
            if (a == (GeneralVector2D) null)
            {
                throw new ArgumentNullException("a");
            }

            if (b == (GeneralVector2D) null)
            {
                throw new ArgumentNullException("b");
            }

            return new GeneralVector2D(a.ToVector() + b.ToVector());
        }

        /// <summary>
        /// Implements the operator -.
        /// </summary>
        /// <param name="a">The first GeneralVector2D.</param>
        /// <param name="b">The second GeneralVector2D.</param>
        /// <returns>The result of the operator.</returns>
        public static GeneralVector2D operator -(GeneralVector2D a, GeneralVector2D b)
        {
            if (a == (GeneralVector2D) null)
            {
                throw new ArgumentNullException("a");
            }

            if (b == (GeneralVector2D) null)
            {
                throw new ArgumentNullException("b");
            }

            return new GeneralVector2D(a.ToVector() - b.ToVector());
        }

        /// <summary>
        /// Implements the operator *.
        /// </summary>
        /// <param name="a">The first GeneralVector2D.</param>
        /// <param name="b">The second GeneralVector2D.</param>
        /// <returns>The result of the operator.</returns>
        public static double operator *(GeneralVector2D a, GeneralVector2D b)
        {
            if (a == (GeneralVector2D) null)
            {
                throw new ArgumentNullException("a");
            }

            if (b == (GeneralVector2D) null)
            {
                throw new ArgumentNullException("b");
            }

            return a.ToVector() * b.ToVector();
        }

        /// <summary>
        /// Implements the operator *.
        /// </summary>
        /// <param name="value">The scalar value with which the vector should be multiplied.</param>
        /// <param name="a">The GeneralVector2D instance.</param>
        /// <returns>The result of the operator.</returns>
        public static GeneralVector2D operator *(double value, GeneralVector2D a)
        {
            if (a == (GeneralVector2D) null)
            {
                throw new ArgumentNullException("a");
            }

            return new GeneralVector2D(value * a.ToVector());
        }

        /// <summary>
        /// Implements the operator *.
        /// </summary>
        /// <param name="a">The GeneralVector2D instance.</param>
        /// <param name="value">The scalar value with which the vector should be multiplied.</param>
        /// <returns>The result of the operator.</returns>
        public static GeneralVector2D operator *(GeneralVector2D a, double value)
        {
            if (a == (GeneralVector2D) null)
            {
                throw new ArgumentNullException("a");
            }

            return new GeneralVector2D(a.ToVector() * value);
        }

        /// <summary>
        /// Converts the GeneralVector2D to a normal n-dimensional vector.
        /// </summary>
        /// <returns>The converted vector.</returns>
        public GeneralVector ToVector()
        {
            return new GeneralVector(this.VectorData);
        }
    }
}