﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SmartMathLibrary.Geometry
{
    /// <summary>
    /// This class describes an arc in a plane space.
    /// </summary>
    [Serializable]
    public class Arc : GeometricObject, IArc
    {
        /// <summary>
        /// The radius of the arc.
        /// </summary>
        private double radius;

        /// <summary>
        /// The center of the arc.
        /// </summary>
        private Point2D center;

        /// <summary>
        /// The angle of the arc.
        /// </summary>
        private double angle;

        /// <summary>
        /// Initializes a new instance of the <see cref="Arc"/> class.
        /// </summary>
        /// <param name="arc">The arc to clone.</param>
        public Arc(Arc arc)
        {
            if (arc == (Arc) null)
            {
                throw new ArgumentNullException("arc");
            }

            this.radius = arc.Radius;
            this.angle = arc.Angle;
            this.center = arc.Center;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Arc"/> class.
        /// </summary>
        /// <param name="radius">The radius of the arc.</param>
        /// <param name="angle">The angle of the arc.</param>
        public Arc(double radius, double angle)
        {
            this.radius = radius;
            this.angle = angle;
            this.center = new Point2D();
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Arc"/> class.
        /// </summary>
        /// <param name="radius">The radius of the arc.</param>
        /// <param name="angle">The angle of the arc.</param>
        /// <param name="center">The center of the arc.</param>
        public Arc(double radius, double angle, Point2D center)
        {
            if (center == (Point2D) null)
            {
                throw new ArgumentNullException("center");
            }

            this.radius = radius;
            this.angle = angle;
            this.center = center;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Arc"/> class.
        /// </summary>
        /// <param name="radius">The radius of the arc.</param>
        /// <param name="angle">The angle of the arc.</param>
        /// <param name="x">The x position of the arc.</param>
        /// <param name="y">The y position of the arc.</param>
        public Arc(double radius, double angle, double x, double y)
        {
            this.radius = radius;
            this.angle = angle;
            this.center = new Point2D(x, y);
        }

        /// <summary>
        /// Gets or sets the radius of the arc.
        /// </summary>
        /// <value>The radius of the arc.</value>
        public double Radius
        {
            get { return radius; }
            set { radius = value; }
        }

        /// <summary>
        /// Gets or sets the center of the arc.
        /// </summary>
        /// <value>The center of the arc.</value>
        public Point2D Center
        {
            get { return center; }
            set { center = value; }
        }

        /// <summary>
        /// Gets or sets the angle of the arc.
        /// </summary>
        /// <value>The angle of the arc.</value>
        public double Angle
        {
            get { return angle; }
            set { angle = value; }
        }

        /// <summary>
        /// Copies the current instance of an arc object.
        /// </summary>
        /// <returns>A copy of the current instance.</returns>
        public Arc Copy()
        {
            return new Arc(this.radius, this.angle, this.center.X, this.center.Y);
        }

        /// <summary>
        /// Implements the operator ==.
        /// </summary>
        /// <param name="a">The first arc.</param>
        /// <param name="b">The second arc.</param>
        /// <returns>The result of the operator.</returns>
        public static bool operator ==(Arc a, Arc b)
        {
            if (System.Object.ReferenceEquals(a, b))
            {
                return true;
            }

            if (((object) a == null) || ((object) b == null))
            {
                return false;
            }

            return ((a.Center == b.Center) && (a.Radius == b.Radius) && (a.Angle == b.Angle));
        }

        /// <summary>
        /// Implements the operator !=.
        /// </summary>
        /// <param name="a">The first arc.</param>
        /// <param name="b">The second arc.</param>
        /// <returns>The result of the operator.</returns>
        public static bool operator !=(Arc a, Arc b)
        {
            return !(a == b);
        }

        /// <summary>
        /// Compares the current instance of an arc to another.
        /// </summary>
        /// <param name="compare">The arc to compare.</param>
        /// <returns>True if the arcs are even otherwise, false.</returns>
        public bool CompareTo(Arc compare)
        {
            if (compare == (Arc) null)
            {
                throw new ArgumentNullException("compare");
            }

            return this == compare;
        }

        /// <summary>
        /// Determines whether the specified <see cref="T:System.Object"/> is equal to the current <see cref="T:System.Object"/>.
        /// </summary>
        /// <param name="obj">The <see cref="T:System.Object"/> to compare with the current <see cref="T:System.Object"/>.</param>
        /// <returns>
        /// true if the specified <see cref="T:System.Object"/> is equal to the current <see cref="T:System.Object"/>; otherwise, false.
        /// </returns>
        /// <exception cref="T:System.NullReferenceException">The <paramref name="obj"/> parameter is null.</exception>
        public override bool Equals(object obj)
        {
            return base.Equals(obj);
        }

        /// <summary>
        /// Serves as a hash function for a particular type.
        /// </summary>
        /// <returns>
        /// A hash code for the current <see cref="T:System.Object"/>.
        /// </returns>
        public override int GetHashCode()
        {
            return base.GetHashCode();
        }

        /// <summary>
        /// Returns a <see cref="T:System.String"/> that represents the current <see cref="T:System.Object"/>.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.String"/> that represents the current <see cref="T:System.Object"/>.
        /// </returns>
        public override string ToString()
        {
            return "Angle = " + this.angle.ToString() + " Radius = " + this.radius.ToString() + " Center = " +
                   this.center.ToString();
        }
    }
}