﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SmartMathLibrary.Geometry
{
    /// <summary>
    /// This class represents a plane line in a two dimensional space.
    /// </summary>
    [Serializable]
    public class Line2D : GeometricObject, ILine2D
    {
        /// <summary>
        /// The start point of the line.
        /// </summary>
        private Point2D startPoint;

        /// <summary>
        /// The endpoint of the line.
        /// </summary>
        private Point2D endPoint;

        /// <summary>
        /// Initializes a new instance of the <see cref="Line2D"/> class.
        /// </summary>
        public Line2D()
        {
            this.startPoint = new Point2D();
            this.endPoint = new Point2D();
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Line2D"/> class.
        /// </summary>
        /// <param name="line">The line to clone.</param>
        public Line2D(Line2D line)
        {
            if (line == (Line2D) null)
            {
                throw new ArgumentNullException("line");
            }

            this.startPoint = line.StartPoint;
            this.endPoint = line.EndPoint;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Line2D"/> class.
        /// </summary>
        /// <param name="startPoint">The start point of the line.</param>
        /// <param name="endPoint">The endpoint of the line.</param>
        public Line2D(Point2D startPoint, Point2D endPoint)
        {
            if (endPoint == (Point2D) null)
            {
                throw new ArgumentNullException("endPoint");
            }
            if (startPoint == (Point2D) null)
            {
                throw new ArgumentNullException("startPoint");
            }

            this.startPoint = startPoint;
            this.endPoint = endPoint;
        }

        /// <summary>
        /// Gets or sets the start point of the line.
        /// </summary>
        /// <value>The start point of the line.</value>
        public Point2D StartPoint
        {
            get { return startPoint; }
            set { startPoint = value; }
        }

        /// <summary>
        /// Gets or sets the endpoint of the line.
        /// </summary>
        /// <value>The endpoint of the line.</value>
        public Point2D EndPoint
        {
            get { return endPoint; }
            set { endPoint = value; }
        }

        /// <summary>
        /// Calculates the length of the line.
        /// </summary>
        /// <returns>The length of the line.</returns>
        public double GetLineLength()
        {
            Point2D tempuri = this.endPoint - this.startPoint;

            return ExtendedMath.Phytagoras(tempuri.X, tempuri.Y);
        }

        /// <summary>
        /// Implements the operator ==.
        /// </summary>
        /// <param name="a">The first line.</param>
        /// <param name="b">The second line.</param>
        /// <returns>The result of the operator.</returns>
        public static bool operator ==(Line2D a, Line2D b)
        {
            if (System.Object.ReferenceEquals(a, b))
            {
                return true;
            }

            if (((object) a == null) || ((object) b == null))
            {
                return false;
            }

            return ((a.StartPoint == b.StartPoint) && (a.EndPoint == b.EndPoint));
        }

        /// <summary>
        /// Implements the operator !=.
        /// </summary>
        /// <param name="a">The first line.</param>
        /// <param name="b">The second line.</param>
        /// <returns>The result of the operator.</returns>
        public static bool operator !=(Line2D a, Line2D b)
        {
            return !(a == b);
        }

        /// <summary>
        /// Compares the current instance of a line with another line object. 
        /// </summary>
        /// <param name="compare">The line to compare.</param>
        /// <returns>True if the position of the lines are even otherwise the method will 
        /// return false.</returns>
        public bool CompareTo(Line2D compare)
        {
            if (compare == (Line2D) null)
            {
                throw new ArgumentNullException("compare");
            }

            return this == compare;
        }

        /// <summary>
        /// Translates the line with the specified parameters.
        /// </summary>
        /// <param name="translateValue">The value with which the line has to be translated.</param>
        public void TranslateLine(double translateValue)
        {
            this.startPoint.TranslateTransformation(translateValue);
            this.endPoint.TranslateTransformation(translateValue);
        }

        /// <summary>
        /// Translates the line with the specified parameters.
        /// </summary>
        /// <param name="translateValueX">The translate value for the x position.</param>
        /// <param name="translateValueY">The translate value for the y position.</param>
        public void TranslateLine(double translateValueX, double translateValueY)
        {
            this.startPoint.TranslateTransformation(translateValueX, translateValueY);
            this.endPoint.TranslateTransformation(translateValueX, translateValueY);
        }

        /// <summary>
        /// Scales the line with a specified factor.
        /// </summary>
        /// <param name="scaleFactor">The scale factor for x and y position.</param>
        public void ScaleLine(double scaleFactor)
        {
            this.startPoint.ScaleTransformation(scaleFactor);
            this.endPoint.ScaleTransformation(scaleFactor);
        }

        /// <summary>
        /// Scales the line with specified factors.
        /// </summary>
        /// <param name="scaleFactorX">The scale factor for the x position.</param>
        /// <param name="scaleFactorY">The scale factor for the y position.</param>
        public void ScaleLine(double scaleFactorX, double scaleFactorY)
        {
            this.startPoint.ScaleTransformation(scaleFactorX, scaleFactorY);
            this.endPoint.ScaleTransformation(scaleFactorX, scaleFactorY);
        }

        /// <summary>
        /// Determines whether the specified <see cref="T:System.Object"/> is equal to the current <see cref="T:System.Object"/>.
        /// </summary>
        /// <param name="obj">The <see cref="T:System.Object"/> to compare with the current <see cref="T:System.Object"/>.</param>
        /// <returns>
        /// true if the specified <see cref="T:System.Object"/> is equal to the current <see cref="T:System.Object"/>; otherwise, false.
        /// </returns>
        /// <exception cref="T:System.NullReferenceException">The <paramref name="obj"/> parameter is null.</exception>
        public override bool Equals(object obj)
        {
            return base.Equals(obj);
        }

        /// <summary>
        /// Serves as a hash function for a particular type.
        /// </summary>
        /// <returns>
        /// A hash code for the current <see cref="T:System.Object"/>.
        /// </returns>
        public override int GetHashCode()
        {
            return base.GetHashCode();
        }

        /// <summary>
        /// Returns a <see cref="T:System.String"/> that represents the current <see cref="T:System.Object"/>.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.String"/> that represents the current <see cref="T:System.Object"/>.
        /// </returns>
        public override string ToString()
        {
            return this.startPoint.ToString() + " | " + this.endPoint.ToString();
        }
    }
}