namespace SmartMathLibrary
{
    /// <summary>
    /// Interface to implement a ComplexMatrix class.
    /// </summary>
    public interface IComplexMatrix
    {
        /// <summary>
        /// Gets the number of columns.
        /// </summary>
        int Columns { get; }

        /// <summary>
        /// Gets the number of rows.
        /// </summary>
        int Rows { get; }

        /// <summary>
        /// Gets the matrix data in form of an array.
        /// </summary>
        ComplexNumber[,] MatrixData { get; }

        /// <summary>
        /// Copies the current instance.
        /// </summary>
        /// <returns>A copy of the current instance.</returns>
        ComplexMatrix Copy();

        /// <summary>
        /// Determines whether the current matrix is a square matrix.
        /// </summary>
        /// <value>
        /// True if the current matrix is a square matrix otherwise, false. 
        /// </value>
        bool IsSquareMatrix { get; }

        /// <summary>
        /// Determines whether the current matrix is a diagonal matrix.
        /// </summary>
        /// <value>
        /// True if the current matrix is a diagonal matrix otherwise, false.
        /// </value>
        bool IsDiagonalMatrix { get; }

        /// <summary>
        /// Determines whether the current matrix is a column vector.
        /// </summary>
        /// <value>
        /// True if the current matrix is a column vector otherwise, false.
        /// </value>
        bool IsColumnVector { get; }

        /// <summary>
        /// Determines whether the current matrix is a row vector.
        /// </summary>
        /// <value>
        /// True if this instance is row vector otherwise, false.
        /// </value>
        bool IsRowVector { get; }

        /// <summary>
        /// Gets the trace of the matrix.
        /// </summary>
        /// <value>The trace of the matrix.</value>
        ComplexNumber Trace { get; }

        /// <summary>
        /// Extracts the real number part of the complex number values into a new matrix.
        /// </summary>
        /// <returns>The real number part of the complex number values.</returns>
        Matrix ExtractRealNumberPartMatrix();

        /// <summary>
        /// Extracts the imaginary number part of the complex number values into a new matrix.
        /// </summary>
        /// <returns>The imaginary number part of the complex number values.</returns>
        Matrix ExtractImaginaryNumberPartMatrix();

        /// <summary>
        /// Creates the inverse matrix of the current matrix.
        /// </summary>
        /// <returns>The created inverse matrix.</returns>
        ComplexMatrix CreateInverseMatrix();

        /// <summary>
        /// Calculates the main diagonal product of the current matrix.
        /// </summary>
        /// <returns>The main diagonal product of the current matrix.</returns>
        ComplexNumber CalculateMainDiagonalProduct();

        /// <summary>
        /// Gets the number at a position of the matrix.
        /// </summary>
        /// <param name="position">The position of the value.</param>
        /// <returns>The number at the specified position.</returns>
        ComplexNumber GetValueAtPosition(MatrixPosition position);

        /// <summary>
        /// Gets the value at the specified position of the matrix.
        /// </summary>
        /// <param name="row">The row index where the value should be set.</param>
        /// <param name="column">The column index where the value should be set.</param>
        /// <returns>The value at the specified position.</returns>
        ComplexNumber GetValueAtPosition(int row, int column);

        /// <summary>
        /// Transposes the current matrix.
        /// </summary>
        /// <returns>The transposed matrix.</returns>
        ComplexMatrix Transpose();

        /// <summary>
        /// Sets the number at a position of the matrix.
        /// </summary>
        /// <param name="position">The position where the value should be set.</param>
        /// <param name="value">The number of the element.</param>
        void SetValueAtPosition(MatrixPosition position, ComplexNumber value);

        /// <summary>
        /// Sets the number at a position of the matrix.
        /// </summary>
        /// <param name="row">The row index of the value to set.</param>
        /// <param name="column">The column index of the value to set.</param>
        /// <param name="value">The value to set.</param>
        void SetValueAtPosition(int row, int column, ComplexNumber value);

        /// <summary>
        /// Compares the current instance of a matrix to another one.
        /// </summary>
        /// <param name="matrix">The matrix to compare.</param>
        /// <returns>True if both matrices are even otherwise, false.</returns>
        bool CompareTo(ComplexMatrix matrix);
    }
}