using System.Collections.Generic;

namespace SmartMathLibrary
{
    /// <summary>
    /// Interface to implement a CubicSpline class.
    /// </summary>
    public interface ICubicSpline
    {
        /// <summary>
        /// Gets or sets the components or parts of the spline.
        /// </summary>
        /// <value>The components or parts of the spline.</value>
        List<CubicSplineComponent> Components { get; }

        /// <summary>
        /// Gets the number of splineparts.
        /// </summary>
        /// <value>The number of splineparts.</value>
        int Count { get; }

        /// <summary>
        /// Adds a specified splinepart to the current spline.
        /// </summary>
        /// <param name="component">The splinepart to add.</param>
        void Add(CubicSplineComponent component);

        /// <summary>
        /// Removes the splinepart at the specified index.
        /// </summary>
        /// <param name="index">The specified index.</param>
        void RemoveAt(int index);

        /// <summary>
        /// Removes all splineparts.
        /// </summary>
        void Clear();

        /// <summary>
        /// Calculates the value of the spline at the specified position x.
        /// </summary>
        /// <param name="x">The specified position x.</param>
        /// <returns>The s(x) of the spline.</returns>
        double SolveAt(double x);

        /// <summary>
        /// Determines whether [the current spline is defined for] [the specified x value].
        /// </summary>
        /// <param name="x">The specified x value, which should be checked.</param>
        /// <returns>
        /// 	<c>true</c> if [the current spline is defined for] [the specified x value];
        /// 	otherwise, <c>false</c>.
        /// </returns>
        bool IsDefinedFor(double x);

        /// <summary>
        /// Creates a copy of the current instance.
        /// </summary>
        /// <returns>A copy of the current instance.</returns>
        CubicSpline Copy();

        /// <summary>
        /// Gets the enumerator.
        /// </summary>
        /// <returns>The enumerator of the current iteration.</returns>
        IEnumerator<CubicSplineComponent> GetEnumerator();

        /// <summary>
        /// Compares the current instance of a cubic spline to another one. 
        /// The order of the splineparts, which the spline includes doesnt matter.
        /// </summary>
        /// <param name="compare">The cubicspline to compare.</param>
        /// <returns>True if the splines are even otherwise, false.</returns>
        bool CompareTo(CubicSpline compare);
    }
}