﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using SmartMathLibrary;
using SmartMathLibrary.LinearEquationSolvers.Preconditioners;

namespace SmartMathLibrary.LinearEquationSolvers
{
    /// <summary>
    /// This class provides the solving of a linear system of equation by using the ConjugateGradientSquared method.
    /// The algorithm, which will be used partly based on the IML++ library.
    /// </summary>
    [Serializable]
    public class ConjugateGradientSquaredSystemSolver : AbstractNonStationaryIterativeMethod
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="ConjugateGradientSquaredSystemSolver"/> class.
        /// </summary>
        /// <param name="coefficientMatrix">The coefficient matrix of the equation system to solve.</param>
        /// <param name="resultVector">The result vector of the equation system to solve.</param>
        /// <param name="preconditioner">The preconditioner which should be used for the iterative method.</param>
        public ConjugateGradientSquaredSystemSolver(Matrix coefficientMatrix, GeneralVector resultVector,
                                                    IPreconditioner preconditioner)
            : base(coefficientMatrix, resultVector, preconditioner)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ConjugateGradientSquaredSystemSolver"/> class.
        /// </summary>
        /// <param name="equationSystem">The equation system to solve.</param>
        /// <param name="preconditioner">The preconditioner which should be used for the iterative method.</param>
        public ConjugateGradientSquaredSystemSolver(LinearEquationSystem equationSystem, IPreconditioner preconditioner)
            : base(equationSystem, preconditioner)
        {
        }

        /// <summary>
        /// Solves the linear system of equation.
        /// </summary>
        /// <param name="startValues">The start values for x.</param>
        /// <param name="iterations">The maximum numbers of iterations to solve the system of equation.</param>
        /// <param name="precision">The precision of the result.</param>
        /// <returns>
        /// The vector which solves the linear system of equation Ax = b.
        /// </returns>
        public override GeneralVector SolveLinearSystem(GeneralVector startValues, int iterations, double precision)
        {
            //The algorithm partly based on the IML++ library of the National Institute of Standards and Technology.
            double residual;
            double rho2 = 0;
            double lenghtB = this.ResultVector.Length;
            GeneralVector p = new GeneralVector(startValues.Count);
            GeneralVector q = new GeneralVector(startValues.Count);
            GeneralVector r = this.ResultVector - this.CoefficientMatrix * startValues;
            GeneralVector rt = r.Copy();
            Matrix preconditionMatrix = this.Preconditioner.ComputePreconditionMatrix();

            if (lenghtB == 0.0)
            {
                lenghtB = 1;
            }

            if ((residual = r.Length / lenghtB) <= precision)
            {
                this.relativeError = residual;
                this.neededIterations = 0;
                this.precisionError = false;

                return startValues;
            }

            for (int i = 0; i < iterations; i++)
            {
                double rho1 = GeneralVector.ComputeDotProduct(rt, r);

                if (rho1 == 0)
                {
                    this.relativeError = r.Length / lenghtB;
                    this.neededIterations = i + 1;
                    this.precisionError = false;

                    return startValues;
                }

                GeneralVector u;

                if (i == 0)
                {
                    u = r;
                    p = u;
                }
                else
                {
                    double beta = rho1 / rho2;

                    u = r + beta * q;
                    p = u + beta * (q + beta * p);
                }

                GeneralVector phat = preconditionMatrix * p;
                GeneralVector vhat = this.CoefficientMatrix * phat;
                double alpha = rho1 / GeneralVector.ComputeDotProduct(rt, vhat);

                q = u - alpha * vhat;
                GeneralVector uhat = preconditionMatrix * (u + q);
                startValues += alpha * uhat;
                GeneralVector qhat = this.CoefficientMatrix * uhat;
                r -= alpha * qhat;
                rho2 = rho1;

                if ((residual = r.Length / lenghtB) < precision)
                {
                    this.relativeError = residual;
                    this.neededIterations = i + 1;
                    this.precisionError = false;

                    return startValues;
                }
            }

            this.relativeError = residual;
            this.neededIterations = iterations;
            this.precisionError = false;

            return startValues;
        }
    }
}