﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using SmartMathLibrary;
using SmartMathLibrary.LinearEquationSolvers;
using SmartMathLibrary.LinearEquationSolvers.Preconditioners;

namespace SmartMathLibrary.LinearEquationSolvers
{
    /// <summary>
    /// This class provides the solving of a linear system of equation by using the ConjugateGradient method.
    /// The algorithm, which will be used partly based on the IML++ library.
    /// </summary>
    [Serializable]
    public class ConjugateGradientSystemSolver : AbstractNonStationaryIterativeMethod
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="ConjugateGradientSystemSolver"/> class.
        /// </summary>
        /// <param name="coefficientMatrix">The coefficient matrix of the equation system to solve.</param>
        /// <param name="resultVector">The result vector of the equation system to solve.</param>
        /// <param name="preconditioner">The preconditioner which should be used for the iterative method.</param>
        public ConjugateGradientSystemSolver(Matrix coefficientMatrix, GeneralVector resultVector,
                                             IPreconditioner preconditioner)
            : base(coefficientMatrix, resultVector, preconditioner)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ConjugateGradientSystemSolver"/> class.
        /// </summary>
        /// <param name="equationSystem">The equation system to solve.</param>
        /// <param name="preconditioner">The preconditioner which should be used for the iterative method.</param>
        public ConjugateGradientSystemSolver(LinearEquationSystem equationSystem, IPreconditioner preconditioner)
            : base(equationSystem, preconditioner)
        {
        }

        /// <summary>
        /// Solves the linear system of equation.
        /// </summary>
        /// <param name="startValues">The start values for x.</param>
        /// <param name="iterations">The maximum numbers of iterations to solve the system of equation.</param>
        /// <param name="precision">The precision of the result.</param>
        /// <returns>
        /// The vector which solves the linear system of equation Ax = b.
        /// </returns>
        public override GeneralVector SolveLinearSystem(GeneralVector startValues, int iterations, double precision)
        {
            //The algorithm partly based on the IML++ library of the National Institute of Standards and Technology.
            double residual;
            double rho1 = 0;
            double lenghtB = this.ResultVector.Length;
            Matrix preconditionMatrix = this.Preconditioner.ComputePreconditionMatrix();
            GeneralVector p = new GeneralVector(startValues.Count);
            GeneralVector r = this.ResultVector - this.CoefficientMatrix * startValues;

            if (lenghtB == 0.0)
            {
                lenghtB = 1;
            }

            if ((residual = r.Length / lenghtB) <= precision)
            {
                this.relativeError = residual;
                this.neededIterations = 0;
                this.precisionError = false;

                return startValues;
            }

            for (int i = 0; i < iterations; i++)
            {
                GeneralVector z = preconditionMatrix * r;
                double rho = GeneralVector.ComputeDotProduct(r, z);

                if (i == 0)
                {
                    p = z;
                }
                else
                {
                    double beta = rho / rho1;

                    p = z + beta * p;
                }

                GeneralVector q = this.CoefficientMatrix * p;
                double alpha = rho / GeneralVector.ComputeDotProduct(p, q);

                startValues += alpha * p;
                r -= alpha * q;

                if ((residual = r.Length / lenghtB) <= precision)
                {
                    this.relativeError = residual;
                    this.neededIterations = i + 1;
                    this.precisionError = false;

                    return startValues;
                }

                rho1 = rho;
            }

            this.relativeError = residual;
            this.neededIterations = iterations;
            this.precisionError = true;

            return startValues;
        }
    }
}