﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using SmartMathLibrary.LinearEquationSolvers;

namespace SmartMathLibrary.LinearEquationSolvers
{
    /// <summary>
    /// This class provides methods to solve a complex linear system by using the elimination algorithm 
    /// of Gauss.
    /// </summary>
    [Serializable]
    public class GaussComplexLinearSystemSolver
    {
        /// <summary>
        /// The complex system of equation to solve.
        /// </summary>
        private readonly ComplexLinearEquationSystem equationSystem;

        /// <summary>
        /// Initializes a new instance of the <see cref="GaussComplexLinearSystemSolver"/> class.
        /// </summary>
        /// <param name="coefficientMatrix">The coefficient matrix of the equation system to solve.</param>
        /// <param name="resultVector">The result vector of the equation system to solve.</param>
        public GaussComplexLinearSystemSolver(ComplexMatrix coefficientMatrix, GeneralComplexVector resultVector)
        {
            this.equationSystem = new ComplexLinearEquationSystem(coefficientMatrix, resultVector);
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="GaussComplexLinearSystemSolver"/> class.
        /// </summary>
        /// <param name="equationSystem">The system of equation to solve.</param>
        public GaussComplexLinearSystemSolver(ComplexLinearEquationSystem equationSystem)
        {
            this.equationSystem = equationSystem;
        }

        /// <summary>
        /// Gets or sets the coefficient matrix of the equation system to solve.
        /// </summary>
        /// <value>The coefficient matrix of the equation system to solve.</value>
        public ComplexMatrix CoefficientMatrix
        {
            get { return equationSystem.CoefficientMatrix; }
            set { equationSystem.CoefficientMatrix = value; }
        }

        /// <summary>
        /// Gets or sets the result vector of the equation system to solve.
        /// </summary>
        /// <value>The result vector of the equation system to solve.</value>
        public GeneralComplexVector ResultVector
        {
            get { return equationSystem.ResultVector; }
            set { equationSystem.ResultVector = value; }
        }

        /// <summary>
        /// Solves the linear system of equation by using the elimination algorithm of Gauss.
        /// </summary>
        /// <returns>The results of the system of equation.</returns>
        public GeneralComplexVector SolveLinearSystem()
        {
            ComplexToRealLinearSystemTransform complexTransform =
                new ComplexToRealLinearSystemTransform(new ComplexLinearEquationSystem(this.CoefficientMatrix,
                                                                                       this.ResultVector));
            GaussLinearSystemSolver gaussSolver =
                new GaussLinearSystemSolver(complexTransform.TransformToRealEquationSystem());
            GeneralVector result = gaussSolver.SolveLinearSystem();


            return AbstractComplexSplittingMethod.AmalgamateComplexVector(result);
        }
    }
}