﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Threading;

namespace SmartMathLibrary.Management.Monitoring
{
    /// <summary>
    /// Defines an abstract class for memory monitoring.
    /// </summary>
    [Serializable]
    public abstract class AbstractMemoryMonitor : IMonitor
    {
        /// <summary>
        /// The message limit.
        /// </summary>
        private double messageLimit;

        /// <summary>
        /// The sampling rate
        /// </summary>
        private double samplingRate;

        /// <summary>
        /// The timer for checking processor usage.
        /// </summary>
        private readonly DispatcherTimer samplingTimer;

        /// <summary>
        /// Defines the priority of the monitor.
        /// </summary>
        private readonly MonitoringPriority monitoringPriority;

        /// <summary>
        /// Initializes a new instance of the <see cref="AbstractMemoryMonitor"/> class.
        /// </summary>
        /// <param name="messageLimit">The message limit. If the message limit get exceed the monitor 
        /// automatically throws a LimitExceedanceEvent.</param>
        /// <param name="samplingRate">The sampling rate.</param>
        protected AbstractMemoryMonitor(double messageLimit, double samplingRate)
        {
            this.samplingRate = samplingRate;
            this.messageLimit = messageLimit;
            this.monitoringPriority = MonitoringPriority.Normal;
            this.samplingTimer = new DispatcherTimer(DispatcherPriority.Normal);

            this.samplingTimer.Interval = TimeSpan.FromMilliseconds(samplingRate);
            this.samplingTimer.Tick += new EventHandler(this.SamplingTimer_Tick);
            this.samplingTimer.Start();
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="AbstractMemoryMonitor"/> class.
        /// </summary>
        /// <param name="messageLimit">The message limit. If the message limit get exceed the monitor 
        /// automatically throws a LimitExceedanceEvent.</param>
        /// <param name="samplingRate">The sampling rate.</param>
        /// <param name="startMonitoring">if set to <c>true</c> the monitor will start automatically
        /// otherwise, it must be started by method Start().</param>
        protected AbstractMemoryMonitor(double messageLimit, double samplingRate, bool startMonitoring)
        {
            this.samplingRate = samplingRate;
            this.messageLimit = messageLimit;
            this.monitoringPriority = MonitoringPriority.Normal;
            this.samplingTimer = new DispatcherTimer(DispatcherPriority.Normal);

            this.samplingTimer.Interval = TimeSpan.FromMilliseconds(samplingRate);
            this.samplingTimer.Tick += new EventHandler(this.SamplingTimer_Tick);

            if (startMonitoring)
            {
                this.samplingTimer.Start();
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="AbstractMemoryMonitor"/> class.
        /// </summary>
        /// <param name="messageLimit">The message limit. If the message limit get exceed the monitor 
        /// automatically throws a LimitExceedanceEvent.</param>
        /// <param name="samplingRate">The sampling rate.</param>
        /// <param name="monitoringPriority">The monitoring priority.</param>
        protected AbstractMemoryMonitor(double messageLimit, double samplingRate, MonitoringPriority monitoringPriority)
        {
            DispatcherPriority tempuri = DispatcherPriority.Normal;

            switch (monitoringPriority)
            {
                case MonitoringPriority.ApplicationIdle:

                    tempuri = DispatcherPriority.ApplicationIdle;
                    break;

                case MonitoringPriority.Background:

                    tempuri = DispatcherPriority.Background;
                    break;

                case MonitoringPriority.SystemIdle:

                    tempuri = DispatcherPriority.SystemIdle;
                    break;

                case MonitoringPriority.Normal:

                    tempuri = DispatcherPriority.Normal;
                    break;
            }

            this.samplingRate = samplingRate;
            this.messageLimit = messageLimit;
            this.monitoringPriority = monitoringPriority;
            this.samplingTimer = new DispatcherTimer(tempuri);

            this.samplingTimer.Interval = TimeSpan.FromMilliseconds(samplingRate);
            this.samplingTimer.Tick += new EventHandler(this.SamplingTimer_Tick);
            this.samplingTimer.Start();
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="AbstractMemoryMonitor"/> class.
        /// </summary>
        /// <param name="messageLimit">The message limit. If the message limit get exceed the monitor 
        /// automatically throws a LimitExceedanceEvent.</param>
        /// <param name="samplingRate">The sampling rate.</param>
        /// <param name="monitoringPriority">The monitoring priority.</param>
        /// <param name="startMonitoring">if set to <c>true</c> the monitor will start automatically
        /// otherwise, it must be started by method Start().</param>
        protected AbstractMemoryMonitor(double messageLimit, double samplingRate, MonitoringPriority monitoringPriority,
                                        bool startMonitoring)
        {
            DispatcherPriority tempuri = DispatcherPriority.Normal;

            switch (monitoringPriority)
            {
                case MonitoringPriority.ApplicationIdle:

                    tempuri = DispatcherPriority.ApplicationIdle;
                    break;

                case MonitoringPriority.Background:

                    tempuri = DispatcherPriority.Background;
                    break;

                case MonitoringPriority.SystemIdle:

                    tempuri = DispatcherPriority.SystemIdle;
                    break;

                case MonitoringPriority.Normal:

                    tempuri = DispatcherPriority.Normal;
                    break;
            }

            this.samplingRate = samplingRate;
            this.messageLimit = messageLimit;
            this.monitoringPriority = monitoringPriority;
            this.samplingTimer = new DispatcherTimer(tempuri);

            this.samplingTimer.Interval = TimeSpan.FromMilliseconds(samplingRate);
            this.samplingTimer.Tick += new EventHandler(this.SamplingTimer_Tick);

            if (startMonitoring)
            {
                this.samplingTimer.Start();
            }
        }

        /// <summary>
        /// Gets the monitoring priority.
        /// </summary>
        /// <value>The monitoring priority.</value>
        public MonitoringPriority MonitoringPriority
        {
            get { return this.monitoringPriority; }
        }

        /// <summary>
        /// Gets or sets the message limit.
        /// </summary>
        /// <value>The message limit.</value>
        public double MessageLimit
        {
            get { return this.messageLimit; }
            set { this.messageLimit = value; }
        }

        /// <summary>
        /// Gets or sets the sampling rate of the monitor.
        /// </summary>
        /// <value>The sampling rate of the monitor.</value>
        public double SamplingRate
        {
            get { return this.samplingRate; }
            set { this.samplingRate = value; }
        }

        /// <summary>
        /// Starts the monitoring.
        /// </summary>
        public void StartMonitoring()
        {
            this.samplingTimer.Start();
        }

        /// <summary>
        /// Stops the monitoring.
        /// </summary>
        public void StopMonitoring()
        {
            this.samplingTimer.Stop();
        }

        /// <summary>
        /// Handles the Tick event of the samplingTimer control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected virtual void SamplingTimer_Tick(object sender, EventArgs e)
        {
        }
    }
}