﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SmartMathLibrary.NonlinearEquationSolvers
{
    /// <summary>
    /// This class provides the finding of roots of a polynomial by using the bisection method.
    /// </summary>
    [Serializable]
    public class BisectionMethodRootFinder : AbstractDerivativeNoNeedRootFinder
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="BisectionMethodRootFinder"/> class.
        /// </summary>
        /// <param name="polynomial">The polynomial for finding the roots.</param>
        public BisectionMethodRootFinder(Polynomial polynomial)
            : base(polynomial)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="BisectionMethodRootFinder"/> class.
        /// </summary>
        /// <param name="polynomial">The polynomial for finding the roots.</param>
        public BisectionMethodRootFinder(SimplePolynomial polynomial)
            : base(polynomial)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="BisectionMethodRootFinder"/> class.
        /// </summary>
        /// <param name="function">The function for finding the roots.</param>
        public BisectionMethodRootFinder(IRealFunction function)
            : base(function)
        {
        }

        /// <summary>
        /// Finds one root of the polynomial by using the bisection method. If the polynomial
        /// has not real roots, which are between the specified values s and t, the result of the
        /// method is wrong. So the result of the method has to checked.
        /// </summary>
        /// <param name="s">The x value for the first starting point. It has to be a value on the left
        /// side of the root to find.</param>
        /// <param name="t">The x value for the second starting point. It has to be a value on the right
        /// side of the root to find.</param>
        /// <returns>
        /// A root of the polynomial, which lies between the specified values s and t.
        /// </returns>
        public double FindRoots(double s, double t)
        {
            return this.FindRoots(s, t, 1e-15, 1000);
        }

        /// <summary>
        /// Finds one root of the polynomial by using the bisection method. If the polynomial
        /// has not real roots, which are between the specified values s and t, the result of the
        /// method is wrong. So the result of the method has to checked.
        /// </summary>
        /// <param name="s">The x value for the first starting point. It has to be a value on the left
        /// side of the root to find.</param>
        /// <param name="t">The x value for the second starting point. It has to be a value on the right
        /// side of the root to find.</param>
        /// <param name="iterations">The maximum of iterations to do, for finding the root.</param>
        /// <returns>
        /// A root of the polynomial, which lies between the specified values s and t.
        /// </returns>
        public double FindRoots(double s, double t, int iterations)
        {
            return this.FindRoots(s, t, 1e-15, iterations);
        }

        /// <summary>
        /// Finds one root of the polynomial by using the bisection method. If the polynomial 
        /// has not real roots, which are between the specified values s and t, the result of the 
        /// method is wrong. So the result of the method has to checked.
        /// </summary>
        /// <param name="s">The x value for the first starting point. It has to be a value on the left 
        /// side of the root to find.</param>
        /// <param name="t">The x value for the second starting point. It has to be a value on the right 
        /// side of the root to find.</param>
        /// <param name="precision">The precision of the result.</param>
        /// <param name="iterations">The maximum of iterations to do, for finding the root.</param>
        /// <returns>A root of the polynomial, which lies between the specified values s and t.</returns>
        public double FindRoots(double s, double t, double precision, int iterations)
        {
            double tempuri;

            for (int i = 0; i < iterations; i++)
            {
                if (Math.Abs(t - s) < precision)
                {
                    this.NeededIterations = i;
                    this.PrecisionError = false;
                    this.RelativeError = Math.Abs(t - s);

                    return (t + s) / 2;
                }

                tempuri = (t + s) / 2;

                if ((this.Function.SolveAt(s)
                     * this.Function.SolveAt(tempuri)) > 0)
                {
                    s = tempuri;
                }
                else
                {
                    t = tempuri;
                }
            }

            this.PrecisionError = true;
            this.NeededIterations = iterations;
            this.RelativeError = Math.Abs(t - s);

            return (t + s) / 2;
        }
    }
}