﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SmartMathLibrary.NonlinearEquationSolvers
{
    /// <summary>
    /// This class provides the finding of roots of a function by using the secant method.
    /// </summary>
    [Serializable]
    public class SecantMethodRootFinder : AbstractDerivativeNoNeedRootFinder
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="SecantMethodRootFinder"/> class.
        /// </summary>
        /// <param name="polynomial">The polynomial for finding the roots.</param>
        public SecantMethodRootFinder(Polynomial polynomial)
            : base(polynomial)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="SecantMethodRootFinder"/> class.
        /// </summary>
        /// <param name="polynomial">The polynomial for finding the roots.</param>
        public SecantMethodRootFinder(SimplePolynomial polynomial)
            : base(polynomial)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="SecantMethodRootFinder"/> class.
        /// </summary>
        /// <param name="function">The function for finding the roots.</param>
        public SecantMethodRootFinder(HardRealFunction function)
            : base(function)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="SecantMethodRootFinder"/> class.
        /// </summary>
        /// <param name="function">The function for finding the roots.</param>
        public SecantMethodRootFinder(IRealFunction function)
            : base(function)
        {
        }

        /// <summary>
        /// Finds one root of the function by using the secant method. If the function
        /// has not real roots, which are between the specified values s and t, the result of the
        /// method is wrong. So the result of the method has to checked.
        /// </summary>
        /// <param name="s">The x value for the first starting point. It has to be a value on the left
        /// side of the root to find.</param>
        /// <param name="t">The x value for the second starting point. It has to be a value on the right
        /// side of the root to find.</param>
        /// <returns>
        /// A root of the function, which lies between the specified values s and t.
        /// </returns>
        public double FindRoots(double s, double t)
        {
            return this.FindRoots(s, t, 1e-15, 1000);
        }

        /// <summary>
        /// Finds one root of the function by using the secant method. If the function
        /// has not real roots, which are between the specified values s and t, the result of the
        /// method is wrong. So the result of the method has to checked.
        /// </summary>
        /// <param name="s">The x value for the first starting point. It has to be a value on the left
        /// side of the root to find.</param>
        /// <param name="t">The x value for the second starting point. It has to be a value on the right
        /// side of the root to find.</param>
        /// <param name="iterations">The maximum of iterations to do, for finding the root.</param>
        /// <returns>
        /// A root of the function, which lies between the specified values s and t.
        /// </returns>
        public double FindRoots(double s, double t, int iterations)
        {
            return this.FindRoots(s, t, 1e-15, iterations);
        }

        /// <summary>
        /// Finds one root of the function by using the secant method. If the function 
        /// has not real roots, which are between the specified values s and t, the result of the 
        /// method is wrong. So the result of the method has to checked.
        /// </summary>
        /// <param name="s">The x value for the first starting point. It has to be a value on the left 
        /// side of the root to find.</param>
        /// <param name="t">The x value for the second starting point. It has to be a value on the right 
        /// side of the root to find.</param>
        /// <param name="precision">The precision of the result.</param>
        /// <param name="iterations">The maximum of iterations to do, for finding the root.</param>
        /// <returns>A root of the function, which lies between the specified values s and t.</returns>
        public double FindRoots(double s, double t, double precision, int iterations)
        {
            double tempuri = 0;

            for (int i = 0; i < iterations; i++)
            {
                tempuri = (s - t) / (this.Function.SolveAt(s)
                                     - this.Function.SolveAt(t)) *
                          this.Function.SolveAt(s);

                if (Math.Abs(tempuri) < precision)
                {
                    this.NeededIterations = i;
                    this.PrecisionError = false;
                    this.RelativeError = Math.Abs(tempuri);

                    return s;
                }

                t = s;
                s = s - tempuri;
            }

            this.PrecisionError = true;
            this.NeededIterations = iterations;
            this.RelativeError = Math.Abs(tempuri);

            return s;
        }
    }
}