﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using SmartMathLibrary;

namespace SmartMathLibrary.NumericalIntegration
{
    /// <summary>
    /// This class provides an integration for linear splines.
    /// </summary>
    [Serializable]
    public class LinearSplineIntegrator : AbstractSplineIntegrator
    {
        /// <summary>
        /// This field holds the spline to integrate.
        /// </summary>
        private IFinityLinearSplineIntegral integral;

        /// <summary>
        /// Initializes a new instance of the <see cref="LinearSplineIntegrator"/> class.
        /// </summary>
        /// <param name="integral">The spline to integrate.</param>
        public LinearSplineIntegrator(IFinityLinearSplineIntegral integral)
        {
            this.integral = integral;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="LinearSplineIntegrator"/> class.
        /// </summary>
        /// <param name="integral">The spline to integrate.</param>
        public LinearSplineIntegrator(FinityLinearSplineIntegral integral)
        {
            if (integral == (FinityLinearSplineIntegral) null)
            {
                throw new ArgumentNullException("integral");
            }

            if (integral.Function == (LinearSpline) null)
            {
                throw new ArgumentNullException("integral");
            }

            this.integral = integral;
        }

        /// <summary>
        /// Gets or sets the spline to integrate.
        /// </summary>
        /// <value>The spline to integrate.</value>
        public IFinityLinearSplineIntegral Integral
        {
            get { return integral; }
            set { integral = value; }
        }


        /// <summary>
        /// Integrates the spline in a numerical way.
        /// </summary>
        /// <returns>The value of the intergral.</returns>
        public override double Integrate()
        {
            bool swapSign = false;
            double a = this.integral.A;
            double b = this.integral.B;

            if (!this.integral.Function.IsDefinedFor(a))
            {
                throw new Exception("The spline is not defined for the integral lower bound a.");
            }

            if (!this.integral.Function.IsDefinedFor(b))
            {
                throw new Exception("The spline is not defined for the integral upper bound b.");
            }

            if (b < a)
            {
                swapSign = true;
                ExtendedMath.Swap(ref a, ref b);
            }

            double result = 0;

            foreach (LinearSplineComponent component in this.integral.Function.Components)
            {
                if ((a <= component.Start.X) && (b >= component.End.X))
                {
                    result += InternalIntegrate(component);
                }
                else
                {
                    if ((component.Start.X < a) && (b < component.End.X))
                    {
                        result += InternalPartIntegrateAToB(component, a, b);
                    }
                    else if ((component.Start.X < a) && (a < component.End.X))
                    {
                        result += InternalPartIntegrateAToEnd(component, a);
                    }
                    else if ((component.Start.X < b) && (b < component.End.X))
                    {
                        result += InternalPartIntegrateStartToB(component, b);
                    }
                }
            }

            if (swapSign)
            {
                return - result;
            }

            return result;
        }

        private static double InternalIntegrate(LinearSplineComponent component)
        {
            SimplePolynomial upgrade = new SimplePolynomial(0, component.LinearFunction.TranslationConstant,
                                                            component.LinearFunction.Slope / 2);

            return upgrade.SolveAt(Math.Max(component.Start.X, component.End.X)) -
                   upgrade.SolveAt(Math.Min(component.Start.X, component.End.X));
        }

        private static double InternalPartIntegrateAToEnd(LinearSplineComponent component, double a)
        {
            SimplePolynomial upgrade = new SimplePolynomial(0, component.LinearFunction.TranslationConstant,
                                                            component.LinearFunction.Slope / 2);

            return upgrade.SolveAt(Math.Max(a, component.End.X)) - upgrade.SolveAt(Math.Min(a, component.End.X));
        }

        private static double InternalPartIntegrateStartToB(LinearSplineComponent component, double b)
        {
            SimplePolynomial upgrade = new SimplePolynomial(0, component.LinearFunction.TranslationConstant,
                                                            component.LinearFunction.Slope / 2);

            return upgrade.SolveAt(Math.Max(component.Start.X, b)) - upgrade.SolveAt(Math.Min(component.Start.X, b));
        }

        private static double InternalPartIntegrateAToB(LinearSplineComponent component, double a, double b)
        {
            SimplePolynomial upgrade = new SimplePolynomial(0, component.LinearFunction.TranslationConstant,
                                                            component.LinearFunction.Slope / 2);

            return upgrade.SolveAt(Math.Max(a, b)) - upgrade.SolveAt(Math.Min(a, b));
        }
    }
}