﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SmartMathLibrary.NumericalIntegration
{
    /// <summary>
    /// This class provides the numerical integration for integrals by using the composite 
    /// Simpson rule.
    /// </summary>
    [Serializable]
    public class SimpsonCompositeIntegrator : AbstractCompositeIntegrator
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="SimpsonCompositeIntegrator"/> class.
        /// </summary>
        /// <param name="a">The lower value a of the integral.</param>
        /// <param name="b">The upper value b of the integral.</param>
        /// <param name="function">The function of the integral.</param>
        public SimpsonCompositeIntegrator(double a, double b, IRealFunction function)
            : base(a, b, function)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="SimpsonCompositeIntegrator"/> class.
        /// </summary>
        /// <param name="a">The lower value a of the integral.</param>
        /// <param name="b">The upper value b of the integral.</param>
        /// <param name="function">The function of the integral.</param>
        public SimpsonCompositeIntegrator(double a, double b, SimplePolynomial function)
            : base(a, b, function)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="SimpsonCompositeIntegrator"/> class.
        /// </summary>
        /// <param name="a">The lower value a of the integral.</param>
        /// <param name="b">The upper value b of the integral.</param>
        /// <param name="function">The function of the integral.</param>
        public SimpsonCompositeIntegrator(double a, double b, Polynomial function)
            : base(a, b, function)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="SimpsonCompositeIntegrator"/> class.
        /// </summary>
        /// <param name="a">The lower value a of the integral.</param>
        /// <param name="b">The upper value b of the integral.</param>
        /// <param name="function">The function of the integral.</param>
        public SimpsonCompositeIntegrator(double a, double b, SimpleRationalPolynomial function)
            : base(a, b, function)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="SimpsonCompositeIntegrator"/> class.
        /// </summary>
        /// <param name="a">The lower value a of the integral.</param>
        /// <param name="b">The upper value b of the integral.</param>
        /// <param name="function">The function of the integral.</param>
        public SimpsonCompositeIntegrator(double a, double b, HardRealFunction function)
            : base(a, b, function)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="SimpsonCompositeIntegrator"/> class.
        /// </summary>
        /// <param name="integral">The integral for the numerical integration.</param>
        public SimpsonCompositeIntegrator(FinitelyDoubleIntegral integral)
            : base(integral.A, integral.B, integral.Function)
        {
        }

        /// <summary>
        /// Integrates the integral in a numerical way. After running the numerical integration
        /// it should be checked if the result has a precision error.
        /// </summary>
        /// <param name="iterations">The maximum numbers of iterations.</param>
        /// <param name="precision">The minimum precision of the numerical integration.</param>
        /// <returns>The area of the specified integral.</returns>
        public override double Integrate(int iterations, double precision)
        {
            bool swapBounds = false;

            if (this.B < this.A)
            {
                double tempuri = this.A;

                this.A = this.B;
                this.B = tempuri;
                swapBounds = true;
            }

            int i = 4;
            int endIterations = 2 * iterations;
            double newValue = 0;
            double oldValue = this.InternalIntegrate(2);

            while (i < endIterations)
            {
                newValue = this.InternalIntegrate(i);
                this.relativeError = Math.Abs(newValue - oldValue);

                if (this.relativeError < precision)
                {
                    this.neededIterations = i / 2;
                    this.precisionError = false;

                    if (swapBounds)
                    {
                        double tempuri = this.A;

                        this.A = this.B;
                        this.B = tempuri;

                        return -newValue;
                    }

                    return newValue;
                }

                oldValue = newValue;
                i += 2;
            }

            this.neededIterations = i / 2;
            this.precisionError = true;

            if (swapBounds)
            {
                double tempuri = this.A;

                this.A = this.B;
                this.B = tempuri;

                return -newValue;
            }

            return newValue;
        }

        /// <summary>
        /// Internal integration algorithm for the used integration method.
        /// </summary>
        /// <param name="parts">The number of parts to divide the area in.</param>
        /// <returns>The integrated area of the specified integral.</returns>
        private double InternalIntegrate(int parts)
        {
            if (ExtendedMath.IsOdd(parts))
            {
                parts += 1;
            }

            double h = (this.B - this.A) / (double) parts;
            double ans = this.Function.SolveAt(this.A) + this.Function.SolveAt(this.B);
            double x = this.A + h;
            double m = 4;

            for (int i = 1; i < parts; i++)
            {
                ans += m * this.Function.SolveAt(x);
                m = 6 - m;
                x = x + h;
            }

            return h * ans / 3;
        }
    }
}