﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SmartMathLibrary.NumericalIntegration
{
    /// <summary>
    /// This class provides the numerical integration for integrals by using the Simpson rule.
    /// </summary>
    [Serializable]
    public class SimpsonIntegrator : AbstractSimpleIntegrator
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="SimpsonIntegrator"/> class.
        /// </summary>
        /// <param name="a">The lower value a of the integral.</param>
        /// <param name="b">The upper value b of the integral.</param>
        /// <param name="polynomial">The polynomial of the integral.</param>
        public SimpsonIntegrator(int a, int b, Polynomial polynomial)
            : base(a, b, polynomial)
        {
        }

        /// <summary>
        /// Integrates the integral in a numerical way. After running the numerical integration
        /// it should be checked if the result has a precision error.
        /// </summary>
        /// <returns>The area of the specified integral.</returns>
        public override double Integrate()
        {
            bool swapBounds = false;

            if (this.B < this.A)
            {
                int tempuri = this.A;

                this.A = this.B;
                this.B = tempuri;
                swapBounds = true;
            }

            double c = (this.A + this.B) / 2.0;
            double h3 = (this.B - this.A) / 6.0;

            this.estimatedError = this.CalculateEstimatedError();

            if (swapBounds)
            {
                int tempuri = this.A;

                this.A = this.B;
                this.B = tempuri;

                return -(h3 * (this.Polynomial.SolveAt(this.A) + 4.0 * this.Polynomial.SolveAt(c) + this.Polynomial.SolveAt(this.B)));
            }

            return h3 *(this.Polynomial.SolveAt(this.A) + 4.0 * this.Polynomial.SolveAt(c) + this.Polynomial.SolveAt(this.B));
        }

        /// <summary>
        /// Calculates the estimated error of the computed area of the integral.
        /// </summary>
        /// <returns>The estimated error of the computed area of the integral.</returns>
        private double CalculateEstimatedError()
        {
            double error = Double.MinValue;

            if (this.Polynomial.Degree >= 4)
            {
                Polynomial polynomial = this.Polynomial.Derivative().Derivative().Derivative().Derivative();

                for (int i = this.A; i <= this.B; i++)
                {
                    double tempuri = polynomial.SolveAt(i);

                    if (error < tempuri)
                    {
                        error = tempuri;
                    }
                }

                return (Math.Pow(this.B - this.A, 5) / 2880) * error;
            }

            return 0;
        }
    }
}