﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SmartMathLibrary.Optimization
{
    /// <summary>
    /// This class represents an abstract layer for simple optimizer classes.
    /// </summary>
    [Serializable]
    public abstract class AbstractSimpleOptimizer
    {
        /// <summary>
        /// Describes if the calculated extremum has a precision error.
        /// </summary>
        protected bool precisionError;

        /// <summary>
        /// The number of needed iteration to find the extremum.
        /// </summary>
        protected int neededIterations;

        /// <summary>
        /// The relative error of the result.
        /// </summary>
        protected double relativeError;

        /// <summary>
        /// The polynomial, from which an extremum should be found.
        /// </summary>
        private Polynomial sourcePolynomial;

        /// <summary>
        /// Initializes a new instance of the <see cref="AbstractSimpleOptimizer"/> class.
        /// </summary>
        /// <param name="sourcePolynomial">The polynomial from which an extremum should be found.</param>
        protected AbstractSimpleOptimizer(Polynomial sourcePolynomial)
        {
            if (sourcePolynomial == (Polynomial) null)
            {
                throw new ArgumentNullException("sourcePolynomial");
            }

            this.sourcePolynomial = sourcePolynomial;
        }

        /// <summary>
        /// Describes if the calculated extremum has a precision error. If true the
        /// calculated extremum may not be correct.
        /// </summary>
        /// <value><c>True</c> if [precision error]; otherwise, <c>false</c>.</value>
        public bool PrecisionError
        {
            get { return precisionError; }
        }

        /// <summary>
        /// The number of needed iteration to find the extremum.
        /// </summary>
        /// <value>The needed iterations to find the extremum.</value>
        public int NeededIterations
        {
            get { return neededIterations; }
        }

        /// <summary>
        /// Gets the relative error of the result.
        /// </summary>
        /// <value>The relative error of the result.</value>
        public double RelativeError
        {
            get { return relativeError; }
        }

        /// <summary>
        /// Gets or sets the polynomial, from which an extremum should be found.
        /// </summary>
        /// <value>The polynomial, from which an extremum should be found.</value>
        public Polynomial SourcePolynomial
        {
            get { return sourcePolynomial; }
            set { sourcePolynomial = value; }
        }
    }
}