﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SmartMathLibrary.Optimization
{
    /// <summary>
    /// This class finds an extremum of a real function. The algorithm based on Brents root finding 
    /// method. 
    /// </summary>
    [Serializable]
    public class BrentMethodOptimizer : AbstractOptimizer
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="BrentMethodOptimizer"/> class.
        /// </summary>
        /// <param name="polynomial">The function from which an extremum should be found.</param>
        public BrentMethodOptimizer(Polynomial polynomial)
            : base(polynomial)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="BrentMethodOptimizer"/> class.
        /// </summary>
        /// <param name="polynomial">The function from which an extremum should be found.</param>
        public BrentMethodOptimizer(SimplePolynomial polynomial)
            : base(polynomial)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="BrentMethodOptimizer"/> class.
        /// </summary>
        /// <param name="function">The function from which an extremum should be found.</param>
        public BrentMethodOptimizer(IRealFunction function)
            : base(function)
        {
        }

        /// <summary>
        /// Finds an extremum of the specified function.
        /// </summary>
        /// <param name="a">The x value for the first starting point. It has to be a value on the left
        /// side of the extremum to find.</param>
        /// <param name="b">The x value for the second starting point. It has to be a value on the right
        /// side of the extremum to find.</param>
        /// <returns>
        /// The value x of the specified function f(x), where the algorithm finds an extremum.
        /// </returns>
        public double FindExtremum(double a, double b)
        {
            return this.FindExtremum(a, b, 1e-15, 1000);
        }

        /// <summary>
        /// Finds an extremum of the specified function.
        /// </summary>
        /// <param name="a">The x value for the first starting point. It has to be a value on the left
        /// side of the extremum to find.</param>
        /// <param name="b">The x value for the second starting point. It has to be a value on the right
        /// side of the extremum to find.</param>
        /// <param name="iterations">The maximum numbers of iterations to find an extremum.</param>
        /// <returns>
        /// The value x of the specified function f(x), where the algorithm finds an extremum.
        /// </returns>
        public double FindExtremum(double a, double b, int iterations)
        {
            return this.FindExtremum(a, b, 1e-15, iterations);
        }

        /// <summary>
        /// Finds an extremum of the specified function.
        /// </summary>
        /// <param name="a">The x value for the first starting point. It has to be a value on the left
        /// side of the extremum to find.</param>
        /// <param name="b">The x value for the second starting point. It has to be a value on the right
        /// side of the extremum to find.</param>
        /// <param name="precision">The precision of the result.</param>
        /// <param name="iterations">The maximum numbers of iterations to find an extremum.</param>
        /// <returns>
        /// The value x of the specified function f(x), where the algorithm finds an extremum.
        /// </returns>
        public double FindExtremum(double a, double b, double precision, int iterations)
        {
            RealDerivativeApproximator approximator = new RealDerivativeApproximator(this.Function);

            int i = 0;
            double fa = approximator.LimitingValueApproximation(a);
            double fb = approximator.LimitingValueApproximation(b);
            double c = a;
            double fc = fa;
            double d = b - a;
            double e = d;

            if (fa * fb > 0)
            {
                throw new ArgumentException("The values at f(a) and f(b) have to have different signs.");
            }

            while (i < iterations)
            {
                if (fb * fc > 0)
                {
                    c = a;
                    fc = fa;
                    d = b - a;
                    e = d;
                }

                if (Math.Abs(fc) < Math.Abs(fb))
                {
                    a = b;
                    b = c;
                    c = a;
                    fa = fb;
                    fb = fc;
                    fc = fa;
                }

                double m = (c - b) / 2;
                double tolerance = 2 * precision * Math.Abs(b);

                if ((Math.Abs(m) > tolerance) && (Math.Abs(fb) > 0))
                {
                    if ((Math.Abs(e) < tolerance) || (Math.Abs(fa) <= Math.Abs(fb)))
                    {
                        d = m;
                        e = m;
                    }
                    else
                    {
                        double q;
                        double p;
                        double s = fb / fa;

                        if (a == c)
                        {
                            p = 2 * m * s;
                            q = 1 - s;
                        }
                        else
                        {
                            double r = fb / fc;

                            q = fa / fc;
                            p = s * (2 * m * q * (q - r) - (b - a) * (r - 1));
                            q = (q - 1) * (r - 1) * (s - 1);
                        }

                        if (p > 0)
                        {
                            q = -q;
                        }
                        else
                        {
                            p = -p;
                        }

                        s = e;
                        e = d;

                        if ((2 * p < 3 * m * q - Math.Abs(tolerance * q)) && (p < Math.Abs(s * q / 2)))
                        {
                            d = p / q;
                        }
                        else
                        {
                            d = m;
                            e = m;
                        }
                    }

                    a = b;
                    fa = fb;

                    if (Math.Abs(d) > tolerance)
                    {
                        b += d;
                    }
                    else
                    {
                        if (m > 0)
                        {
                            b += tolerance;
                        }
                        else
                        {
                            b -= tolerance;
                        }
                    }
                }
                else
                {
                    break;
                }

                fb = approximator.LimitingValueApproximation(b);
                i++;
            }

            this.neededIterations = i;
            this.precisionError = i >= iterations;
            this.relativeError = Math.Abs((c - b) / 2);

            return b;
        }
    }
}