﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SmartMathLibrary
{
    /// <summary>
    /// This class represents a three dimensional point with x,y and a z component.
    /// </summary>
    [Serializable]
    public class Point3D : IPoint3D
    {
        /// <summary>
        /// The x position of the point.
        /// </summary>
        private double x;

        /// <summary>
        /// The y position of the point.
        /// </summary>
        private double y;

        /// <summary>
        /// The z position of the point.
        /// </summary>
        private double z;

        /// <summary>
        /// Initializes a new instance of the <see cref="Point3D"/> class.
        /// </summary>
        public Point3D()
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Point3D"/> class.
        /// </summary>
        /// <param name="point">The point to clone.</param>
        public Point3D(Point3D point)
        {
            if (point == (Point3D) null)
            {
                throw new ArgumentNullException("point");
            }

            this.x = point.X;
            this.y = point.Y;
            this.z = point.Z;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Point3D"/> class.
        /// </summary>
        /// <param name="vector">The vector, which includes the positioninformations.</param>
        public Point3D(GeneralVector3D vector)
        {
            if (vector == (GeneralVector3D) null)
            {
                throw new ArgumentNullException("vector");
            }

            this.x = vector.X;
            this.y = vector.Y;
            this.z = vector.Z;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Point3D"/> class.
        /// </summary>
        /// <param name="x">The x position.</param>
        /// <param name="y">The y position.</param>
        /// <param name="z">The z position.</param>
        public Point3D(double x, double y, double z)
        {
            this.x = x;
            this.y = y;
            this.z = z;
        }

        /// <summary>
        /// Gets or sets the x position.
        /// </summary>
        /// <value>The x position.</value>
        public double X
        {
            get { return x; }
            set { x = value; }
        }

        /// <summary>
        /// Gets or sets the y position.
        /// </summary>
        /// <value>The y position.</value>
        public double Y
        {
            get { return y; }
            set { y = value; }
        }

        /// <summary>
        /// Gets or sets the z position.
        /// </summary>
        /// <value>The z position.</value>
        public double Z
        {
            get { return z; }
            set { z = value; }
        }

        /// <summary>
        /// Compares the current instance with another object of the same type.
        /// </summary>
        /// <param name="obj">An object to compare with this instance.</param>
        /// <returns>
        /// A 32-bit signed integer that indicates the relative order of the objects being compared. The return value has these meanings: Value Meaning Less than zero This instance is less than <paramref name="obj"/>. Zero This instance is equal to <paramref name="obj"/>. Greater than zero This instance is greater than <paramref name="obj"/>.
        /// </returns>
        /// <exception cref="T:System.ArgumentException">
        /// 	<paramref name="obj"/> is not the same type as this instance. </exception>
        public int CompareTo(Object obj)
        {
            if (obj is Point3D)
            {
                if (this.x == ((Point3D) obj).X)
                {
                    return 0;
                }

                if (this.x < ((Point3D) obj).X)
                {
                    return -1;
                }

                if (this.x > ((Point3D) obj).X)
                {
                    return 1;
                }
            }

            return 0;
        }

        /// <summary>
        /// Adds a x, y and z value to the current point instance.
        /// </summary>
        /// <param name="xValue">The x value.</param>
        /// <param name="yValue">The y value.</param>
        /// <param name="zValue">The y value.</param>
        public void Add(double xValue, double yValue, double zValue)
        {
            this.x += xValue;
            this.y += yValue;
            this.z += zValue;
        }

        /// <summary>
        /// Inverts the position elements x y and z.
        /// </summary>
        public void InvertPositionElements()
        {
            this.x = -this.x;
            this.y = -this.y;
            this.z = -this.z;
        }

        /// <summary>
        /// Copies this current instance of the point.
        /// </summary>
        /// <returns>A copy of the current instance.</returns>
        public Point3D Copy()
        {
            return new Point3D(this.x, this.y, this.z);
        }

        /// <summary>
        /// Scales the position elements x, y and z with a specified factor.
        /// </summary>
        /// <param name="scaleFactor">The scale factor.</param>
        public void ScaleTransformation(double scaleFactor)
        {
            this.x *= scaleFactor;
            this.y *= scaleFactor;
            this.z *= scaleFactor;
        }

        /// <summary>
        /// Scales the position elements x, y and z with a specified factor.
        /// </summary>
        /// <param name="scaleFactorX">The scale factor for the x position.</param>
        /// <param name="scaleFactorY">The scale factor for the y position.</param>
        /// <param name="scaleFactorZ">The scale factor for the z position.</param>
        public void ScaleTransformation(double scaleFactorX, double scaleFactorY, double scaleFactorZ)
        {
            this.x *= scaleFactorX;
            this.y *= scaleFactorY;
            this.z *= scaleFactorZ;
        }

        /// <summary>
        /// Translates the point position.
        /// </summary>
        /// <param name="translateValue">The translate value for the x-, y- and z position.</param>
        public void TranslateTransformation(double translateValue)
        {
            this.x += translateValue;
            this.y += translateValue;
            this.z += translateValue;
        }

        /// <summary>
        /// Translates the point position.
        /// </summary>
        /// <param name="translateValueX">The translate value for x position.</param>
        /// <param name="translateValueY">The translate value for y position.</param>
        /// <param name="translateValueZ">The translate value for z position.</param>
        public void TranslateTransformation(double translateValueX, double translateValueY, double translateValueZ)
        {
            this.x += translateValueX;
            this.y += translateValueY;
            this.z += translateValueZ;
        }

        /// <summary>
        /// Calculates the distance from the current instance of a point to another point.
        /// </summary>
        /// <param name="point">The other point.</param>
        /// <returns>The distance from the current instance of a point to another point</returns>
        public double GetDistance(Point3D point)
        {
            if (point == (Point3D) null)
            {
                throw new ArgumentNullException("point");
            }

            Point3D tempuri = this - point;

            return ExtendedMath.Phytagoras(tempuri.X, tempuri.Y, tempuri.Z);
        }

        /// <summary>
        /// Determines whether the position elements are zero.
        /// </summary>
        /// <returns>
        /// 	<c>true</c> if the position elements are zero; otherwise, <c>false</c>.
        /// </returns>
        public bool IsZero()
        {
            return ((this.x == 0) && (this.y == 0) && (this.z == 0));
        }

        /// <summary>
        /// Implements the operator +.
        /// </summary>
        /// <param name="a">The first point.</param>
        /// <param name="b">The second point.</param>
        /// <returns>The result of the operator.</returns>
        public static Point3D operator +(Point3D a, Point3D b)
        {
            if (a == (Point3D) null)
            {
                throw new ArgumentNullException("a");
            }

            if (b == (Point3D) null)
            {
                throw new ArgumentNullException("b");
            }

            Point3D result = new Point3D();

            result.X = a.X + b.X;
            result.Y = a.Y + b.Y;
            result.Z = a.Z + b.Z;

            return result;
        }

        /// <summary>
        /// Implements the operator -.
        /// </summary>
        /// <param name="a">The first point.</param>
        /// <param name="b">The second point.</param>
        /// <returns>The result of the operator.</returns>
        public static Point3D operator -(Point3D a, Point3D b)
        {
            if (a == (Point3D) null)
            {
                throw new ArgumentNullException("a");
            }

            if (b == (Point3D) null)
            {
                throw new ArgumentNullException("b");
            }

            Point3D result = new Point3D();

            result.X = a.X - b.X;
            result.Y = a.Y - b.Y;
            result.Z = a.Z - b.Z;

            return result;
        }

        /// <summary>
        /// Implements the operator *.
        /// </summary>
        /// <param name="a">The first point.</param>
        /// <param name="b">The second point.</param>
        /// <returns>The result of the operator.</returns>
        public static Point3D operator *(Point3D a, Point3D b)
        {
            if (a == (Point3D) null)
            {
                throw new ArgumentNullException("a");
            }

            if (b == (Point3D) null)
            {
                throw new ArgumentNullException("b");
            }

            Point3D result = new Point3D();

            result.X = a.X * b.X;
            result.Y = a.Y * b.Y;
            result.Z = a.Z * b.Z;

            return result;
        }

        /// <summary>
        /// Implements the operator /.
        /// </summary>
        /// <param name="a">The first point.</param>
        /// <param name="b">The second point.</param>
        /// <returns>The result of the operator.</returns>
        public static Point3D operator /(Point3D a, Point3D b)
        {
            if (a == (Point3D) null)
            {
                throw new ArgumentNullException("a");
            }

            if (b == (Point3D) null)
            {
                throw new ArgumentNullException("b");
            }

            Point3D result = new Point3D();

            result.X = a.X / b.X;
            result.Y = a.Y / b.Y;
            result.Z = a.Z / b.Z;

            return result;
        }

        /// <summary>
        /// Implements the operator ==.
        /// </summary>
        /// <param name="a">The first point.</param>
        /// <param name="b">The second point.</param>
        /// <returns>The result of the operator.</returns>
        public static bool operator ==(Point3D a, Point3D b)
        {
            if (System.Object.ReferenceEquals(a, b))
            {
                return true;
            }

            if (((object) a == null) || ((object) b == null))
            {
                return false;
            }

            return ((a.X == b.X) && (a.Y == b.Y) && (a.Z == b.Z));
        }

        /// <summary>
        /// Implements the operator !=.
        /// </summary>
        /// <param name="a">The first point.</param>
        /// <param name="b">The second point.</param>
        /// <returns>The result of the operator.</returns>
        public static bool operator !=(Point3D a, Point3D b)
        {
            return !(a == b);
        }

        /// <summary>
        /// Implements the operator &lt;.
        /// </summary>
        /// <param name="a">The first point.</param>
        /// <param name="b">The second point.</param>
        /// <returns>The result of the operator.</returns>
        public static bool operator <(Point3D a, Point3D b)
        {
            if (a == (Point3D) null)
            {
                throw new ArgumentNullException("a");
            }

            if (b == (Point3D) null)
            {
                throw new ArgumentNullException("b");
            }

            return ((a.X < b.X) && (a.Y < b.Y) && (a.Z < b.Z));
        }

        /// <summary>
        /// Implements the operator &lt;=.
        /// </summary>
        /// <param name="a">The first point.</param>
        /// <param name="b">The second point.</param>
        /// <returns>The result of the operator.</returns>
        public static bool operator <=(Point3D a, Point3D b)
        {
            if (a == (Point3D) null)
            {
                throw new ArgumentNullException("a");
            }

            if (b == (Point3D) null)
            {
                throw new ArgumentNullException("b");
            }

            return ((a.X <= b.X) && (a.Y <= b.Y) && (a.Z <= b.Z));
        }

        /// <summary>
        /// Implements the operator &gt;.
        /// </summary>
        /// <param name="a">The first point.</param>
        /// <param name="b">The second point.</param>
        /// <returns>The result of the operator.</returns>
        public static bool operator >(Point3D a, Point3D b)
        {
            if (a == (Point3D) null)
            {
                throw new ArgumentNullException("a");
            }

            if (b == (Point3D) null)
            {
                throw new ArgumentNullException("b");
            }

            return ((a.X > b.X) && (a.Y > b.Y) && (a.Z > b.Z));
        }

        /// <summary>
        /// Implements the operator &gt;=.
        /// </summary>
        /// <param name="a">The first point.</param>
        /// <param name="b">The second point.</param>
        /// <returns>The result of the operator.</returns>
        public static bool operator >=(Point3D a, Point3D b)
        {
            if (a == (Point3D) null)
            {
                throw new ArgumentNullException("a");
            }

            if (b == (Point3D) null)
            {
                throw new ArgumentNullException("b");
            }

            return ((a.X >= b.X) && (a.Y >= b.Y) && (a.Z >= b.Z));
        }

        /// <summary>
        /// Serves as a hash function for a particular type.
        /// </summary>
        /// <returns>
        /// A hash code for the current <see cref="T:System.Object"/>.
        /// </returns>
        public override int GetHashCode()
        {
            return base.GetHashCode();
        }

        /// <summary>
        /// Determines whether the specified <see cref="T:System.Object"/> is equal to the current <see cref="T:System.Object"/>.
        /// </summary>
        /// <param name="obj">The <see cref="T:System.Object"/> to compare with the current <see cref="T:System.Object"/>.</param>
        /// <returns>
        /// true if the specified <see cref="T:System.Object"/> is equal to the current <see cref="T:System.Object"/>; otherwise, false.
        /// </returns>
        /// <exception cref="T:System.NullReferenceException">The <paramref name="obj"/> parameter is null.</exception>
        public override bool Equals(object obj)
        {
            return base.Equals(obj);
        }

        /// <summary>
        /// Transforms the current point instance into a matrix.
        /// </summary>
        /// <returns>The transformed matrix.</returns>
        public Matrix ToMatrix()
        {
            Matrix matrix = new Matrix(3, 1);

            matrix.SetValueAtPosition(new MatrixPosition(0, 0), this.x);
            matrix.SetValueAtPosition(new MatrixPosition(1, 0), this.y);
            matrix.SetValueAtPosition(new MatrixPosition(2, 0), this.z);

            return matrix;
        }

        /// <summary>
        /// Transforms the current point instance into a vector.
        /// </summary>
        /// <returns>The transformed vector.</returns>
        public GeneralVector ToVector()
        {
            GeneralVector vector = new GeneralVector(3);

            vector.SetComponent(0, this.x);
            vector.SetComponent(1, this.y);
            vector.SetComponent(2, this.z);

            return vector;
        }

        /// <summary>
        /// Returns a <see cref="T:System.String"/> that represents the current <see cref="T:System.Object"/>.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.String"/> that represents the current <see cref="T:System.Object"/>.
        /// </returns>
        public override string ToString()
        {
            return this.x.ToString() + "; " + this.y.ToString() + "; " + this.z.ToString();
        }
    }
}