﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SmartMathLibrary
{
    /// <summary>
    /// This class represents an convergence test for fix point problems like F(x) = f(x) + x.
    /// </summary>
    [Serializable]
    public class RealIterativeConvergenceTest : AbstractConvergenceTest
    {
        /// <summary>
        /// The function, which should be checked.
        /// </summary>
        private IRealFunction function;

        /// <summary>
        /// Initializes a new instance of the <see cref="RealIterativeConvergenceTest"/> class.
        /// </summary>
        /// <param name="function">The function, which should be checked.</param>
        public RealIterativeConvergenceTest(IRealFunction function)
        {
            if (function == (IRealFunction) null)
            {
                throw new ArgumentNullException("function");
            }

            this.function = function;
        }

        /// <summary>
        /// Gets or sets the function, which should be checked.
        /// </summary>
        /// <value>The function, which should be checked.</value>
        public IRealFunction Function
        {
            get { return function; }
            set { function = value; }
        }

        /// <summary>
        /// Checks the convergence of the specified fix point problem.
        /// </summary>
        /// <param name="startValue">The start values for the convergence check.</param>
        /// <param name="iterations">The maximn number of iterations.</param>
        /// <param name="precision">The precision of the result.</param>
        /// <returns>The fix point, which was find by the test.</returns>
        public double CheckConvergence(double startValue, int iterations, double precision)
        {
            double oldValue = Double.MinValue;

            for (int i = 0; i < iterations; i++)
            {
                oldValue = startValue;
                startValue = this.function.SolveAt(startValue);

                if (Math.Abs(oldValue - startValue) < precision)
                {
                    this.convergence = true;
                    this.neededIterations = i + 1;
                    this.relativeError = Math.Abs(oldValue - startValue);

                    return startValue;
                }
            }

            this.convergence = false;
            this.neededIterations = iterations;
            this.relativeError = Math.Abs(oldValue - startValue);

            return startValue;
        }
    }
}