﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SmartMathLibrary.Space.CoreTypes
{
    /// <summary>
    /// This class represents a matrix.
    /// </summary>
    [Serializable]
    public class SpaceMatrix : ISpaceEntry
    {
        /// <summary>
        /// The specified maxtrix data.
        /// </summary>
        [SpaceAttribute] 
        private readonly double[,] data;

        /// <summary>
        /// Initializes a new instance of the <see cref="SpaceMatrix"/> class.
        /// </summary>
        /// <param name="rows">The number of rows of the matrix.</param>
        /// <param name="columns">The number of columns of the matrix.</param>
        public SpaceMatrix(int rows, int columns)
        {
            this.data = new double[rows,columns];
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="SpaceMatrix"/> class.
        /// </summary>
        /// <param name="data">The specified maxtrix data.</param>
        public SpaceMatrix(double[,] data)
        {
            this.data = (double[,]) data.Clone();
        }

        /// <summary>
        /// Gets the number of rows of the matrix.
        /// </summary>
        /// <value>The number of rows of the matrix.</value>
        public int Rows
        {
            get { return this.data.Length; }
        }

        /// <summary>
        /// Gets the number of columns of the matrix.
        /// </summary>
        /// <value>The number of columns of the matrix.</value>
        public int Columns
        {
            get { return this.data.GetLength(1); }
        }

        /// <summary>
        /// Gets or sets the <see cref="System.Double"/> at the specified row and column.
        /// </summary>
        /// <value>The specified value at the specified row and column.</value>
        public double this[int row, int column]
        {
            get { return this.data[row, column]; }
            set { this.data[row, column] = value; }
        }

        /// <summary>
        /// Converts the current space type to a <see cref="SmartMathLibrary.Matrix"/>.
        /// </summary>
        /// <returns>The converted matrix.</returns>
        public Matrix ToMatrix()
        {
            return new Matrix((double[,])this.data.Clone());
        }

        /// <summary>
        /// Creates a new object that is a copy of the current instance.
        /// </summary>
        /// <returns>
        /// A new object that is a copy of this instance.
        /// </returns>
        public object Clone()
        {
            return CloneFunctions.Clone(this);
        }

        /// <summary>
        /// Compares the current <see cref="ISpaceEntry"/> to the specified <see cref="ISpaceEntry"/>.
        /// </summary>
        /// <param name="entry">The specified entry to compare.</param>
        /// <returns>
        /// If the object instances are even, the method returns true otherwise, false.
        /// </returns>
        public bool Equals(ISpaceEntry entry)
        {
            if (entry is SpaceMatrix)
            {
                SpaceMatrix check = (SpaceMatrix)entry;

                if ((this.Rows==check.Rows)&&(this.Columns==check.Columns))
                {
                    for (int i=0;i < check.Rows;i++)
                    {
                        for (int j = 0; j < check.Columns; j++)
                        {
                           if( this[i,j]!=check[i,j])
                           {
                               return false;
                           }
                        }
                    }

                    return true;
                }
            }

            return false;
        }

        /// <summary>
        /// Compares the current <see cref="ISpaceEntry"/> to the specified <see cref="ISpaceEntry"/>.
        /// </summary>
        /// <param name="entry">The specified entry to compare.</param>
        /// <returns>
        /// If the specified object instance is lower than the current instance the method returns true otherwise,
        /// false.
        /// </returns>
        public bool LowerThan(ISpaceEntry entry)
        {
            return false;
        }

        /// <summary>
        /// Compares the current <see cref="ISpaceEntry"/> to the specified <see cref="ISpaceEntry"/>.
        /// </summary>
        /// <param name="entry">The specified entry to compare.</param>
        /// <returns>
        /// If the specified object instance is greater than the current instance the method returns true otherwise,
        /// false.
        /// </returns>
        public bool GreaterThan(ISpaceEntry entry)
        {
            return false;
        }
    }
}