﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SmartMathLibrary.Statistics.Distribution
{
    /// <summary>
    /// This class represents a Laplace distribution.
    /// </summary>
    [Serializable]
    public class LaplaceDistribution
    {
        /// <summary>
        /// The scale parameter of the distribution.
        /// </summary>
        private readonly double scale;

        /// <summary>
        /// The location parameter of the distribution.
        /// </summary>
        private readonly double location;

        /// <summary>
        /// Initializes a new instance of the <see cref="LaplaceDistribution"/> class.
        /// </summary>
        /// <param name="scale">The scale parameter of the distribution.</param>
        /// <param name="location">The location parameter of the distribution.</param>
        public LaplaceDistribution(double location, double scale)
        {
            if (scale <= 0)
            {
                throw new ArgumentException("scale <= 0");
            }

            this.scale = scale;
            this.location = location;
        }

        /// <summary>
        /// Gets the scale parameter of the distribution.
        /// </summary>
        /// <value>The scale parameter of the distribution.</value>
        public double Scale
        {
            get { return this.scale; }
        }

        /// <summary>
        /// Gets the location parameter of the distribution.
        /// </summary>
        /// <value>The location parameter of the distribution.</value>
        public double Location
        {
            get { return this.location; }
        }

        /// <summary>
        /// Gets the mean of the distribution.
        /// </summary>
        /// <value>The mean of the distribution.</value>
        public double Mean
        {
            get { return this.location; }
        }

        /// <summary>
        /// Gets the median of the distribution.
        /// </summary>
        /// <value>The median of the distribution.</value>
        public double Median
        {
            get { return this.location; }
        }

        /// <summary>
        /// Gets the mode of the distribution.
        /// </summary>
        /// <value>The mode of the distribution.</value>
        public double Mode
        {
            get { return this.location; }
        }

        /// <summary>
        /// Gets the variance of the distribution.
        /// </summary>
        /// <value>The variance of the distribution.</value>
        public double Variance
        {
            get { return 2*this.scale*this.scale; }
        }

        /// <summary>
        /// Gets the skewness of the distribution.
        /// </summary>
        /// <value>The skewness of the distribution.</value>
        public double Skewness
        {
            get { return 0; }
        }

        /// <summary>
        /// Computes the probability mass function of the Laplace distribution at the specified postion x.
        /// </summary>
        /// <param name="x">The specified position x.</param>
        /// <returns>The probability mass function of the Laplace distribution at the specified postion x.</returns>
        public double ProbabilityMassFunction(double x)
        {
            return (1/(2*this.scale))*Math.Exp(-(Math.Abs(x - this.location)/this.scale));
        }

        /// <summary>
        /// Computes the cumulative distribution function of the Laplace distribution at the specified postion x.
        /// </summary>
        /// <param name="x">The specified position x.</param>
        /// <returns>The cumulative distribution function of the Laplace distribution at the specified postion x.</returns>
        public double CumulativeDistributionFunction(double x)
        {
            if (x < this.location)
            {
                return (1/(2*this.scale))*(Math.Exp(-((this.location - x)/this.scale)));
            }

            return (1/(2*this.scale))*(Math.Exp(-((x - this.location)/this.scale)));
        }
    }
}