﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using SmartMathLibrary.InternalRoutines;

namespace SmartMathLibrary.Statistics.RandomNumberGenerator
{
    /// <summary>
    /// Generates random deviates from the gamma distribution whose density is (A**R)/Gamma(R) * X**(R-1) * Exp(-A*X).
    /// </summary>
    [Serializable]
    public class GammaDistributionRandomGenerator : AbstractRandomGenerator
    {
        /// <summary>
        /// The location parameter of Gamma distribution.
        /// </summary>
        private readonly double a;

        /// <summary>
        /// The shape parameter of Gamma distribution.
        /// </summary>
        private readonly double r;

        /// <summary>
        /// Internal linker to the base routine.
        /// </summary>
        private readonly StatisticalRandomRoutines provider;

        /// <summary>
        /// Initializes a new instance of the <see cref="GammaDistributionRandomGenerator"/> class.
        /// </summary>
        /// <param name="a">The location parameter of Gamma distribution.</param>
        /// <param name="r">The shape parameter of Gamma distribution.</param>
        public GammaDistributionRandomGenerator(double a, double r)
            : base((uint) DateTime.Now.Millisecond)
        {
            if (a == 0)
            {
                throw new ArgumentException("The parameter a has to be != 0.");
            }

            if (r == 0)
            {
                throw new ArgumentException("The parameter r has to be != 0.");
            }

            this.a = a;
            this.r = r;
            this.provider = new StatisticalRandomRoutines((int) this.Seed);
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="GammaDistributionRandomGenerator"/> class.
        /// </summary>
        /// <param name="a">The location parameter of Gamma distribution.</param>
        /// <param name="r">The shape parameter of Gamma distribution.</param>
        /// <param name="seed">A number used to calculate a starting value for the pseudo-random number sequence.
        /// If a negative number is specified, the absolute value of the number is used.</param>
        public GammaDistributionRandomGenerator(double a, double r, uint seed)
            : base(seed)
        {
            if (a == 0)
            {
                throw new ArgumentException("The parameter a has to be != 0.");
            }

            if (r == 0)
            {
                throw new ArgumentException("The parameter r has to be != 0.");
            }

            this.a = a;
            this.r = r;
            this.provider = new StatisticalRandomRoutines((int) seed);
        }

        /// <summary>
        /// Returns a nonnegative random number.
        /// </summary>
        /// <returns>
        /// A 32-bit signed integer greater than or equal to zero and less than maximum integer value.
        /// </returns>
        public override int Next()
        {
            return this.Next(0, int.MaxValue);
        }

        /// <summary>
        /// Nexts the specified max value.
        /// </summary>
        /// <param name="maxValue">Returns a nonnegative random number less than the specified maximum.</param>
        /// <returns>
        /// A 32-bit signed integer greater than or equal to zero and less than maximum integer value.
        /// </returns>
        public override int Next(int maxValue)
        {
            return this.Next(0, maxValue);
        }

        /// <summary>
        /// Returns a random number within a specified range.
        /// </summary>
        /// <param name="minValue">The inclusive lower bound of the random number returned.</param>
        /// <param name="maxValue">The exclusive upper bound of the random number returned. maxValue must be
        /// greater than or equal to minValue.</param>
        /// <returns>
        /// A 32-bit signed integer greater than or equal to zero and less than maximum integer value.
        /// </returns>
        public override int Next(int minValue, int maxValue)
        {
            if (minValue < 0)
            {
                minValue = 0;
            }

            if (maxValue == int.MaxValue)
            {
                maxValue -= 1;
            }

            if (minValue > maxValue)
            {
                int tempuri = maxValue;

                maxValue = minValue;
                minValue = tempuri;
            }

            double number = this.GenerateRandomNumber();

            if (number < 0)
            {
                number = Math.Abs(number);
            }

            if (number > 1)
            {
                number = number - Math.Truncate(number);
            }

            return (int) (Math.Floor((maxValue - minValue + 1)*number + minValue));
        }

        /// <summary>
        /// Returns a random number between 0.0 and 1.0.
        /// </summary>
        /// <returns>
        /// A double-precision floating point number greater than or equal to 0.0, and less
        /// than 1.0.
        /// </returns>
        public override double NextDouble()
        {
            return this.GenerateRandomNumber();
        }

        private double GenerateRandomNumber()
        {
            return this.provider.gengam(this.a, this.r);
        }
    }
}