﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SmartMathLibrary.Statistics.RandomNumberGenerator
{
    /// <summary>
    /// This class provides the generation of random numbers by using the linear congruential method.
    /// </summary>
    [Serializable]
    public class LinearCongruentialGenerator : AbstractRandomGenerator
    {
        /// <summary>
        /// Internal field, which holds a value to generate the next random number.
        /// </summary>
        private uint nextRandom;

        /// <summary>
        /// Initializes a new instance of the <see cref="LinearCongruentialGenerator"/> class.
        /// </summary>
        public LinearCongruentialGenerator()
            : base((uint) DateTime.Now.Millisecond)
        {
            this.nextRandom = this.Seed;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="LinearCongruentialGenerator"/> class.
        /// </summary>
        /// <param name="seed">A number used to calculate a starting value for the pseudo-random number sequence.
        /// If a negative number is specified, the absolute value of the number is used.</param>
        public LinearCongruentialGenerator(uint seed)
            : base(seed)
        {
            this.nextRandom = this.Seed;
        }

        /// <summary>
        /// Returns a nonnegative random number.
        /// </summary>
        /// <returns>
        /// A 32-bit signed integer greater than or equal to zero and less than 32768 (2^15).
        /// </returns>
        public override int Next()
        {
            return (int) this.GenerateRandomNumber();
        }

        /// <summary>
        /// Nexts the specified max value.
        /// </summary>
        /// <param name="maxValue">Returns a nonnegative random number less than the specified maximum.</param>
        /// <returns>
        /// A 32-bit signed integer greater than or equal to zero and less than 32768 (2^15).
        /// </returns>
        public override int Next(int maxValue)
        {
            return this.Next(0, maxValue);
        }

        /// <summary>
        /// Returns a random number within a specified range.
        /// </summary>
        /// <param name="minValue">The inclusive lower bound of the random number returned.</param>
        /// <param name="maxValue">The exclusive upper bound of the random number returned. maxValue must be
        /// greater than or equal to minValue.</param>
        /// <returns>
        /// A 32-bit signed integer greater than or equal to zero and less than 32768 (2^15).
        /// </returns>
        public override int Next(int minValue, int maxValue)
        {
            if (minValue > maxValue)
            {
                int tempuri = maxValue;

                maxValue = minValue;
                minValue = tempuri;
            }

            return (int) (Math.Floor((maxValue - minValue + 1)*this.NextDouble() + minValue));
        }

        /// <summary>
        /// Returns a random number between 0.0 and 1.0.
        /// </summary>
        /// <returns>
        /// A double-precision floating point number greater than or equal to 0.0, and less
        /// than 1.0.
        /// </returns>
        public override double NextDouble()
        {
            return this.GenerateRandomNumber()/32768.0;
        }

        private uint GenerateRandomNumber()
        {
            this.nextRandom = this.nextRandom*1103515245 + 12345;

            return ((this.nextRandom/65536)%32768);
        }
    }
}