﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using SmartMathLibrary.InternalRoutines;

namespace SmartMathLibrary.Statistics.RandomNumberGenerator
{
    /// <summary>
    /// This class provides the generation of random numbers by using the Ziggurat RNOR method.
    /// </summary>
    [Serializable]
    public class ZigguratRNORRandomGenerator : AbstractRandomGenerator
    {
        /// <summary>
        /// Internal routine class to compute the random numbers.
        /// </summary>
        private readonly ZigguratRandomRoutine baseRoutine;

        /// <summary>
        /// Initializes a new instance of the <see cref="ZigguratRNORRandomGenerator"/> class.
        /// </summary>
        public ZigguratRNORRandomGenerator()
            : base((uint) DateTime.Now.Millisecond)
        {
            this.baseRoutine = new ZigguratRandomRoutine();
            this.baseRoutine.Zigset(this.Seed);
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ZigguratRNORRandomGenerator"/> class.
        /// </summary>
        /// <param name="seed">A number used to calculate a starting value for the pseudo-random number sequence.
        /// If a negative number is specified, the absolute value of the number is used.</param>
        public ZigguratRNORRandomGenerator(uint seed)
            : base(seed)
        {
            this.baseRoutine = new ZigguratRandomRoutine();
            this.baseRoutine.Zigset(seed);
        }

        /// <summary>
        /// Returns a nonnegative random number.
        /// </summary>
        /// <returns>
        /// A 32-bit signed integer greater than or equal to zero and less than 32768 (2^15).
        /// </returns>
        public override int Next()
        {
            return this.Next(0, 32768);
        }

        /// <summary>
        /// Nexts the specified max value.
        /// </summary>
        /// <param name="maxValue">Returns a nonnegative random number less than the specified maximum.</param>
        /// <returns>
        /// A 32-bit signed integer greater than or equal to zero and less than 32768 (2^15).
        /// </returns>
        public override int Next(int maxValue)
        {
            return this.Next(0, maxValue);
        }

        /// <summary>
        /// Returns a random number within a specified range.
        /// </summary>
        /// <param name="minValue">The inclusive lower bound of the random number returned.</param>
        /// <param name="maxValue">The exclusive upper bound of the random number returned. maxValue must be
        /// greater than or equal to minValue.</param>
        /// <returns>
        /// A 32-bit signed integer greater than or equal to zero and less than 32768 (2^15).
        /// </returns>
        public override int Next(int minValue, int maxValue)
        {
            if (minValue > maxValue)
            {
                int tempuri = maxValue;

                maxValue = minValue;
                minValue = tempuri;
            }

            return (int) (Math.Floor((maxValue - minValue + 1)*this.NextDouble() + minValue));
        }

        /// <summary>
        /// Returns a random number between 0.0 and 1.0.
        /// </summary>
        /// <returns>
        /// A double-precision floating point number greater than or equal to 0.0, and less
        /// than 1.0.
        /// </returns>
        public override double NextDouble()
        {
            double tempuri = this.baseRoutine.Nfix();

            return Math.Abs(tempuri - Math.Truncate(tempuri));
        }
    }
}