﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SmartMathLibrary
{
    /// <summary>
    /// This class represents a Taylor series.
    /// </summary>
    [Serializable]
    public class TaylorSeries : ITaylorSeries
    {
        /// <summary>
        /// The constant value of a Taylor series
        /// </summary>
        private double constantValue;

        /// <summary>
        /// The other components of the specified Taylor series.
        /// </summary>
        private readonly List<TaylorSeriesComponent> components;

        /// <summary>
        /// Initializes a new instance of the <see cref="TaylorSeries"/> class.
        /// </summary>
        public TaylorSeries()
        {
            this.components = new List<TaylorSeriesComponent>();
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="TaylorSeries"/> class.
        /// </summary>
        /// <param name="components">The components of the specified Taylor series.</param>
        /// <param name="constantValue">The constant value of the specified Taylor series.</param>
        public TaylorSeries(List<TaylorSeriesComponent> components, double constantValue)
        {
            if (components == (List<TaylorSeriesComponent>) null)
            {
                throw new ArgumentNullException("components");
            }

            this.constantValue = constantValue;
            this.components = components;
        }

        /// <summary>
        /// Gets or sets the components of the Taylor series.
        /// </summary>
        /// <value>The components of the Taylor series.</value>
        public List<TaylorSeriesComponent> Components
        {
            get { return components; }
        }

        /// <summary>
        /// Gets or sets the constant value of the Taylor series.
        /// </summary>
        /// <value>The constant value of the Taylor series.</value>
        public double ConstantValue
        {
            get { return constantValue; }
            set { constantValue = value; }
        }

        /// <summary>
        /// Solves the Taylor series at the specified position x.
        /// </summary>
        /// <param name="x">The specified position x.</param>
        /// <returns>The value p(x) for the specified x value.</returns>
        public double SolveAt(double x)
        {
            double result = this.constantValue;

            foreach (TaylorSeriesComponent component in this.components)
            {
                result += component.SolveAt(x);
            }

            return result;
        }
    }
}